/****************************************************************************
  Copyright (c) 2006, Dell Inc
  All rights reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 
  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 
 
 
*****************************************************************************/
/****************************************************************************
  Notes:
    To support 16 bit and Option ROM users of the library, no ANSI function
    calls are allowed except strcpy, strcat and strlen.  Also no
    memory allocations are allowed.
****************************************************************************/

/****************************************************************************
 Includes this file is dependent on
****************************************************************************/

#include <ipmitool/cssstat.h>
#include <ipmitool/cssipmi.h>
#include <ipmitool/cssapi.h>
#include <ipmitool/csssupt.h>

/****************************************************************************
    Local defines
****************************************************************************/

#define SEL_STANDARD_RECORD_TYPE   (2)
#define SEL_OEM_RECORD_TYPE        (0xC0)
#define SEL_NO_TIME_RECORD_TYPE    (0xE0)
#define SEL_MIN_VALID_TIME_STAMP   (0x20000000)
#define SEL_MAX_VALID_TIME_STAMP   (0x80000000)
#define PCI_IS_SLOT_MASK           (0x80)
#define PCI_SLOT_BUS_NUM_MASK      (0x7F)
#define PCI_DEVICE_NUM_MASK        (0xF8)
#define PCI_FUNCTION_NUM_MASK      (0x07)
#define OEM_BYTE_2_MASK            (0xC0)
#define OEM_BYTE_2                 (0x80)
#define OEM_BYTE_3_MASK            (0x30)
#define OEM_BYTE_3                 (0x20)
#define OEM_BYTES_23_MASK          (OEM_BYTE_2_MASK | OEM_BYTE_3_MASK)
#define OEM_BYTES_23               (OEM_BYTE_2 | OEM_BYTE_3)
#define OEM_ID_DELL                (0x02A2)
#define IPMI_1_5                   (0x51)
#define IPMI_2_0                   (0x02)
#define NUMA_DIMM_SET_MASK         (0x0F)
#define NUMA_DIMM_NODE_MASK        (0xF0)
#define IPMI_OFFSET_MASK           (0x0F)
#define OFFSET_BYTE_6              (0x06)
#define OFFSET_BYTE_1              (0x01)

/****************************************************************************
    Extern Data
****************************************************************************/

/* from SelTxt.c */
extern char g_PciSlot[];
extern char g_Drive[];
extern char g_Slot[];
extern char g_OemEventRecord[];
extern char g_SoftwareEvent[];
extern char g_sensor[];
extern char g_sensorfor[];
extern char g_systemboot[];
extern char g_wasAsserted[];
extern char g_wasDeasserted[];
extern char g_oemEvent[];
extern char g_DefaultUnknownStatus[];
extern char g_Normal[];
extern char g_Warning[];
extern char g_Failure[];
extern char g_DetectedNR[];
extern char g_whileOn[];
extern char g_whileOff[];

extern AppendStateElement g_GenericTable[];
extern unsigned char g_GenericTableSize;
extern AppendStateElement g_SpecificTable[];
extern unsigned char g_SpecificTableSize;
extern PostCodeType g_PostMessages[];
extern unsigned char g_PostMessagesSize;


/****************************************************************************
  Local Data
****************************************************************************/
static CSLFUSERAPI USERAPIList = {0,0,0};

/* Set for DELL PE2900 */
static DECODING_METHOD CSLFMethod = 
{0x00,0x20,0x00,0x00,0x00,0x02,0x00,{0xA2,0x02,0x00},{0x01,0x00},{0x00,0x00,0x00,0x00}};

static char DaysOfMonth[12] = 
{
  31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
};

static const char * g_SELMonthTable[] =
{"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};

static const char * g_DayOfWeekTable[] =
{"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};

static int mytime[] = 
{0, 3, 2, 5, 0, 3, 5, 1, 4, 6, 2, 4};

static int is_little_endian = 1;

#define IS_LITTLE_ENDIAN (*(unsigned char *)&is_little_endian == 1)

/****************************************************************************
  Local Function Prototypes
****************************************************************************/
static int GetSelDateString(
  _IN     const void*     pIPMISelEntry,
  _OUT    char*           pDateNameBuf,
  _INOUT  unsigned short* pDateNameBufSize);

static void GetSensorStatusStr(
  _IN const SELType*       pIPMISelEntry, 
  _IN const SDRType FAR *  pIPMISdr,
  _INOUT char*             statusStr,
  _IN int                  size,
  _OUT unsigned char*      pSeverity);

static SDRType FAR *SelFindSDRRecord(
  _IN const void* pIPMISelEntry,
  _IN void*       puserParameter);

static void GetMemoryDIMMStr(
  _INOUT char*      memStr,
  _IN unsigned char data1,
  _IN unsigned char data2, 
  _IN unsigned char data3);

static void GetPCIInfoStr(
  _INOUT char*      memStr,
  _IN unsigned char data1,
  _IN unsigned char data2, 
  _IN unsigned char data3);

static void GetExtentedHWConfigStr(
  _INOUT char*      configStr,
  _IN unsigned char data1,
  _IN unsigned char data2, 
  _IN unsigned char data3);

static void GetExtentedWD2Str(
  _INOUT char*      memStr,
  _IN unsigned char data1,
  _IN unsigned char data2);

static void GetExtentLinkTuningStr(
  _INOUT char*      memStr, 
  _IN unsigned char data1, 
  _IN unsigned char eventData2,
  _IN unsigned char eventData3);

static void GetPowerSupplyWatts(
  _INOUT char*      memStr, 
  _IN unsigned char data1, 
  _IN unsigned char eventData2,
  _IN unsigned char eventData3);

static void GetPowerSupplyExtend(
  _INOUT char*      memStr, 
  _IN unsigned char data1, 
  _IN unsigned char eventData2,
  _IN unsigned char eventData3);

/*  Code Start */

/****************************************************************************
  CSLFSELUnixToCTime
****************************************************************************/

int CSLFSELUnixToCTime(_IN char*  pUnixStr, _INOUT char*  pCtime)
{
  longdiv_t   value;
  int         month;
  int         year;
  int         day;
  int         dow;
  char        szDayOfWeek[4];
  char        szYear[5];
  char        szMonth[5];
  char        szDay[3];
  char        szHour[3];
  char        szMinute[3];
  char        szSecond[3];

  int status = COMMON_STATUS_BAD_PARAMETER;

  /* check params */
  if (( NULL != pUnixStr) && (NULL != pCtime ))
  {
    pCtime[0] = 0;
    /* Check if <System Boot> by checking first element is a number */
    if ((pUnixStr[0] < '0') || (pUnixStr[0] > '9'))
    {
      strcpy(pCtime, pUnixStr);
    }
    else
    {
      /* parse Unix time that is in current format YYYYMMDDHHMMSS */
      memcpy( szYear,   pUnixStr,    4 );
      szYear[4] = 0;   
      memcpy( szMonth,  pUnixStr+4,  2 );
      szMonth[2] = 0;
      memcpy( szDay,    pUnixStr+6,  2 );
      szDay[2] = 0;
      memcpy( szHour,   pUnixStr+8,  2 );
      szHour[2] = 0;
      memcpy( szMinute, pUnixStr+10, 2 );
      szMinute[2] = 0;
      memcpy( szSecond, pUnixStr+12, 2 );
      szSecond[2] = 0;

      /* convert string number of month to month string */
      month = (int)CSSAsciiToLong(szMonth);
      szMonth[0] = 0;
      strcpy(szMonth, g_SELMonthTable[month-1]);

      /* Hard part is to find day of week */
      year = (int)CSSAsciiToLong(szYear);
      day  = (int)CSSAsciiToLong(szDay);
      if (month < 3)
      {
        year--;
      }
      dow   = (year + year/4 - year/100 + year/400 + mytime[month-1] + day);
      value = CSSLongDiv((long)dow, 7L);
      dow   = (int)value.rem;
      strcpy(szDayOfWeek, g_DayOfWeekTable[dow]);

      /* format to Tue May 1 14:25:03 1984 */
      strcpy(pCtime, szDayOfWeek);
      strcat(pCtime, " ");
      strcat(pCtime, szMonth);
      strcat(pCtime, " ");
      strcat(pCtime, szDay);
      strcat(pCtime, " ");
      strcat(pCtime, szHour);
      strcat(pCtime, ":");
      strcat(pCtime, szMinute);
      strcat(pCtime, ":");
      strcat(pCtime, szSecond);
      strcat(pCtime, " ");
      strcat(pCtime, szYear);
    } 
    status = COMMON_STATUS_SUCCESS;  
  }
  return status;
}


/****************************************************************************
  CSLFSELEntryToStr
****************************************************************************/
int CSLFSELEntryToStr(
  _IN     const void*     pIPMISelEntry,
  _IN     unsigned char   sensorNameGenPolicy,
  _OUT    char*           pLogTimeStr,
  _INOUT  unsigned short* pLogTimeStrSize,
  _OUT    char*           pEventDescStr,
  _INOUT  unsigned short* pEventDescStrSize,
  _OUT    unsigned char*  pSeverity,
  _IN     void*           puserParameter)
{
  unsigned short  strSize;
  unsigned char   instance;
  char            probeName[SENSOR_NAME_STR_SIZE];
  char            entityName[SENSOR_NAME_STR_SIZE];
  char*           sensorTypeString;
  char            sensorStatusString[EVENT_DESC_STR_SIZE];
  void FAR *      pIPMISdr    = NULL;
  void FAR *      pIPMIFruSdr = NULL;    
  int             status      = COMMON_STATUS_BAD_PARAMETER;
  IPMISELEntry*   pSel        = (IPMISELEntry*)pIPMISelEntry;

  UNREFERENCED_PARAMETER(sensorNameGenPolicy);

  probeName[0]  = 0;
  entityName[0] = 0;
  sensorStatusString[0] = 0;

  /* check for mandatory parameters */
  if (NULL != pIPMISelEntry)
  {
    /* compute or generate the time string */
    if ((NULL != pLogTimeStr) && (NULL != pLogTimeStrSize))
    {
      status = GetSelDateString(
        pIPMISelEntry, 
        pLogTimeStr, 
        pLogTimeStrSize);
    }

    /* get event description */
    if ((NULL != pEventDescStr) && (NULL != pEventDescStrSize))
    {
      /* reset return status */
      status = COMMON_STATUS_DATA_OVERRUN;

      /* check to see if this is a OEM record */
      if (SEL_STANDARD_RECORD_TYPE != pSel->recordType)
      {
        if (NULL != pSeverity)
        {
          *pSeverity = CSS_SEVERITY_NORMAL;
        }
        strSize    = (unsigned short)strlen(g_OemEventRecord) + 1;
        if (strSize < *pEventDescStrSize)
        {
          strcpy(pEventDescStr, g_OemEventRecord);
          status = COMMON_STATUS_SUCCESS; 
        }
      }
      else
      {
        /* get sensor type name */
        sensorTypeString = CSSGetSensorTypeStr(
          pSel->sensorType, 
          (pSel->eventDirType & (unsigned char)0x7f));

        /* Get sensor name form SDR */
        pIPMISdr = SelFindSDRRecord(
          pIPMISelEntry, 
          puserParameter);

        if (NULL != pIPMISdr)
        {
          /* instance is for sensors with shared SDRs  */
          instance = pSel->sensorNum;
          instance = instance - 
            CSSSDRGetAttribute(
              pIPMISdr, 
              SDR_ATTRIBUTE_SENSOR_NUMBER,
              USERAPIList.Oem2IPMISDR);

          CSSGetProbeName(
            (IPMISDR FAR *)pIPMISdr, 
            instance, 
            probeName, 
            SENSOR_NAME_STR_SIZE,
            USERAPIList.Oem2IPMISDR);

          /* Get name of the owner of this SDR  */
          pIPMIFruSdr = CSSFindEntitySDRRecord(
            USERAPIList.GetFirstSDR,
            USERAPIList.GetNextSDR,
            USERAPIList.Oem2IPMISDR,
            pIPMISdr, 
            puserParameter);

          if (NULL != pIPMIFruSdr)
          {
            CSSGetProbeName(
              (IPMISDR FAR *)pIPMIFruSdr, 
              0, 
              entityName, 
              SENSOR_NAME_STR_SIZE,
              USERAPIList.Oem2IPMISDR);
          }

          strcpy (sensorStatusString, "undefined status");
          /* now get sensor status */
          GetSensorStatusStr(
            pIPMISelEntry, 
            pIPMISdr, 
            sensorStatusString, 
            EVENT_DESC_STR_SIZE, 
            pSeverity);                 

          if (NULL != pIPMIFruSdr && entityName[0] != 0)
          {
            strSize = (unsigned short)strlen(probeName) + 
              (unsigned short)(2* strlen(entityName)) + 
              (unsigned short)strlen(sensorTypeString) + 
              (unsigned short)strlen(sensorStatusString);
            if (strSize < *pEventDescStrSize)
            {
              /* NOTE: the following code is to 
                 support users of the library
                 that do not have an sprintf function */
              strcpy(pEventDescStr, entityName);
              strcat(pEventDescStr, " ");
              strcat(pEventDescStr, probeName);
              strcat(pEventDescStr, ": ");
              strcat(pEventDescStr, sensorTypeString);
              strcat(pEventDescStr, g_sensorfor);
              strcat(pEventDescStr, entityName);
              strcat(pEventDescStr, ", ");
              strcat(pEventDescStr, sensorStatusString);
              status = COMMON_STATUS_SUCCESS;
            }
          }
          else
          {
            strSize = (unsigned short)strlen(probeName) + 
              (unsigned short)strlen(sensorTypeString) +  
              (unsigned short)strlen(sensorStatusString);
            if (strSize < *pEventDescStrSize)
            {
              /* NOTE: the following code is to 
                 support users of the library
                that do not have an sprintf function */                       
              strcpy(pEventDescStr, probeName);
              strcat(pEventDescStr, ": ");
              strcat(pEventDescStr, sensorTypeString);
              strcat(pEventDescStr, g_sensor);
              strcat(pEventDescStr, ", ");
              strcat(pEventDescStr, sensorStatusString);
              status = COMMON_STATUS_SUCCESS;
            }
          }
        }
        else
        {
          GetSensorStatusStr(
            pIPMISelEntry, 
            pIPMISdr, 
            sensorStatusString, 
            EVENT_DESC_STR_SIZE, 
            pSeverity);

          if (sensorStatusString[0] == 0)
          {
            strcpy(sensorStatusString, g_DefaultUnknownStatus);
          }

          strSize = (unsigned short)(strlen(g_SoftwareEvent) + strlen(sensorTypeString)+
            strlen(g_sensor) + strlen(sensorStatusString) + 3);
          if (strSize < *pEventDescStrSize)
          {
            strcpy(pEventDescStr, g_SoftwareEvent);
            strcat(pEventDescStr, ": ");
            strcat(pEventDescStr, sensorTypeString);
            strcat(pEventDescStr, g_sensor);
            strcat(pEventDescStr, ", ");
            strcat(pEventDescStr, sensorStatusString);
            status = COMMON_STATUS_SUCCESS;
          }
        }
      }
    }
  }
  return status;
}


/****************************************************************************
    CSLFAttach
****************************************************************************/
void CSLFAttach(CSLFUSERAPI *pfnApiList)
{
  USERAPIList.GetFirstSDR = pfnApiList->GetFirstSDR;
  USERAPIList.GetNextSDR  = pfnApiList->GetNextSDR;
  USERAPIList.Oem2IPMISDR = pfnApiList->Oem2IPMISDR;
}


/****************************************************************************
    CSLFDetach
****************************************************************************/
void CSLFDetach(void)
{
  return;
}

/****************************************************************************
    CSLFAttach
****************************************************************************/
int CSLFGetDecodingMethod(_INOUT DECODING_METHOD* method)
{
  int status = COMMON_STATUS_BAD_PARAMETER;
  if (NULL != method)
  {
    memcpy((void*)method, (void*)&CSLFMethod, sizeof(DECODING_METHOD));
    status = COMMON_STATUS_SUCCESS;
  }
  return status;
}

/****************************************************************************
    CSLFAttach
****************************************************************************/
int CSLFSetDecodingMethod(_INOUT DECODING_METHOD* method)
{
  int status = COMMON_STATUS_BAD_PARAMETER;
  if (NULL != method)
  {
    memcpy((void*)&CSLFMethod, (void*)method, sizeof(DECODING_METHOD));
    status = COMMON_STATUS_SUCCESS;
  }
  return status;
}

/****************************************************************************
    GetDateString
****************************************************************************/
static int GetSelDateString(
  _IN     const void*     pIPMISelEntry,
  _OUT    char*           pDateNameBuf,
  _INOUT  unsigned short* pDateNameBufSize)

{
  longdiv_t       value;
  int             seconds;
  int             hours;
  int             minutes;
  int             year;
  int             month;
  int             day;
  int             fourYrBlocks;
  unsigned int    daysPerYr;
  unsigned long   timeval;
  char            numStr[16];  /* The largest number of digits in a long */

  /* is buffer big enough? */
  if (*pDateNameBufSize < (LOG_DATE_STR_SIZE))
  {
    return COMMON_STATUS_DATA_OVERRUN;
  }

  if (((IPMISELEntry *)pIPMISelEntry)->recordType >= SEL_NO_TIME_RECORD_TYPE)
  {
    strcpy(pDateNameBuf, "Not Applicable");
  }
  else
  { 
    timeval = *((unsigned long*)((IPMISELEntry *)pIPMISelEntry)->timeStamp);

    if (IS_LITTLE_ENDIAN == 0)
    { 
      /* swap (little-endian) byte order of timestamp */
      timeval = (((timeval & 0xff) << 24) 
                 + (((timeval >> 8) & 0xff) << 16)
                 + (((timeval >> 16) & 0xff) << 8)
                 + (((timeval >> 24) & 0xff) ));
    }

    /* check for invalid time stamp */
    if ((timeval <= SEL_MIN_VALID_TIME_STAMP) ||
        (timeval >= SEL_MAX_VALID_TIME_STAMP))
    {
      strcpy(pDateNameBuf, g_systemboot);
    }
    else
    {
      /* NOTE: the following code does the same thing as the gmtime function
          but is here to support users that do not have this function */               

      /* get remaining seconds. NOTE: some 16 bit compliers do not support
         32 bit operators like % */
      /* get remaining seconds */
      value   = CSSLongDiv((long)timeval,60L);
      timeval = value.quot;
      seconds = (int)value.rem;
      /* get remaning munites */
      value   = CSSLongDiv((long)timeval,60L);
      timeval = value.quot;
      minutes = (int)value.rem;
      /* get remaining hours */
      value   = CSSLongDiv((long)timeval,24L);
      timeval = value.quot;
      hours   = (int)value.rem;
      /* get number of 4 year blocks. 
         Number of days in 4 years = (365*3)+366 */
      fourYrBlocks = (int)(timeval/1461);
      /* timeval is now number of days since last 4 year block */
      value   = CSSLongDiv((long)timeval,1461L);
      timeval = value.rem;
      /* year of last 4 year block */
      year = (fourYrBlocks<<2) + 1970;  
      /* get corrected year */
      for (;;)
      {
        daysPerYr = 366;
        /* if not a leap year subtract a day */
        if (year & 3)
        {
          daysPerYr--;
        }
        if (timeval < daysPerYr)
        {
          break;
        }
        year++;
        timeval = timeval-daysPerYr;            
      } /* at end of loop, timeval is days from start of year (zero base) */
      timeval++;
      month = 0;
      /* if a leap year then change DayOfMonth array */
      if ((year & 3) == 0)
      {
        DaysOfMonth[1] = 29;
      }
      for (month = 0; (int)DaysOfMonth[month] < (int)timeval; month++)
      {
        timeval = timeval-DaysOfMonth[month];
      }
      day = (int)(timeval);
      /* reset Feb to 28 days */
      DaysOfMonth[1] = 28;
      /* Month is 1 base */
      month++;
      /* generate DellDateName string;
         DellDateName string has the 
         format: yyyymmddhhMMss.uuuuuu+ccc */
      /* NOTE: the following code is to support users of the library
          that do not have an sprintf function */
      CSSlongToAscii((long)year,numStr,10,0);
      strcpy(pDateNameBuf,numStr);
      CSSlongToAscii((long)month,numStr,10,0);
      if (month < 10)
      {
        strcat(pDateNameBuf,"0");
      }
      strcat(pDateNameBuf,numStr);
      CSSlongToAscii((long)day,numStr,10,0);
      if (day < 10)
      {
        strcat(pDateNameBuf,"0");
      }
      strcat(pDateNameBuf,numStr);
      CSSlongToAscii((long)hours,numStr,10,0);
      if (hours < 10)
      {
        strcat(pDateNameBuf,"0");
      }
      strcat(pDateNameBuf,numStr);
      CSSlongToAscii((long)minutes,numStr,10,0);
      if (minutes < 10)
      {
        strcat(pDateNameBuf,"0");
      }
      strcat(pDateNameBuf,numStr);
      CSSlongToAscii((long)seconds,numStr,10,0);
      if (seconds < 10)
      {
        strcat(pDateNameBuf,"0");
      }
      strcat(pDateNameBuf,numStr);
      strcat(pDateNameBuf,".000000");
    }
  }
  return COMMON_STATUS_SUCCESS;
}

/****************************************************************************
     GetSensorStatusStr
****************************************************************************/
static void GetSensorStatusStr(
  _IN const SELType*      pIPMISelEntry, 
  _IN const SDRType FAR * pIPMISdr,
  _INOUT char*            statusStr,
  _IN int                 size,
  _OUT unsigned char*     pSeverity)
{ 
  unsigned char readingType;
  unsigned char deasserted;
  unsigned char eventData1;
  unsigned char eventData2;
  unsigned char eventData3;
  unsigned char sensorType;
  unsigned char index;
  unsigned char count;
  unsigned char thrReadMask = 0;
  unsigned char data1;
  char          memStr[100];
  char*         tmpStr      = NULL;
  IPMISELEntry* pSelEntry   = (IPMISELEntry*)pIPMISelEntry;

  UNREFERENCED_PARAMETER(size);

  memStr[0]   = 0;
  readingType = (unsigned char)(pSelEntry->eventDirType & 0x7f); 
  deasserted  = (unsigned char)(pSelEntry->eventDirType >> 7);   
  eventData1  = (unsigned char)(pSelEntry->eventData1 & 0x0f);
  data1       = (unsigned char)(pSelEntry->eventData1);
  eventData2  = pSelEntry->eventData2; 
  eventData3  = pSelEntry->eventData3; 
  sensorType  = pSelEntry->sensorType;

  if (NULL != pIPMISdr)
  {
    thrReadMask = CSSSDRGetAttribute(
      pIPMISdr, 
      SDR_ATTRIBUTE_THRES_READ_MASK, 
      USERAPIList.Oem2IPMISDR);
  }

  /* set defaults */
  if (NULL != pSeverity)
  {
    *pSeverity = CSS_SEVERITY_UNKNOWN;
  }
  strcpy(statusStr, g_DefaultUnknownStatus);

  /* Generic Sensor reading type */
  if ((readingType > (unsigned char)0) && 
      (readingType < (unsigned char)MAX_READING_TYPES))
  {
    /* unique case for threshold */
    if (IPMI_READING_TYPE_THRESHOLD == readingType)
    {
      if (FALSE == deasserted) /* asserted */
      {
        switch (eventData1)
        {
          case IPMI_UC_HIGH:
          case IPMI_UNR_LOW:
          case IPMI_LNR_HIGH:
          case IPMI_LC_LOW:
            if (NULL != pSeverity)
            {
              *pSeverity = CSS_SEVERITY_CRITICAL;
            }
            strcpy(statusStr,  g_Failure);                            
            break;

          case IPMI_UNC_HIGH:
          case IPMI_UC_LOW:
          case IPMI_LNC_LOW:
          case IPMI_LC_HIGH:
            if (NULL != pSeverity)
            {
              *pSeverity = CSS_SEVERITY_WARNING;
            }
            strcpy(statusStr, g_Warning);
            break;

          case IPMI_UNR_HIGH:
          case IPMI_LNR_LOW:
            if (NULL != pSeverity)
            {
              *pSeverity = CSS_SEVERITY_NONRECOVERABLE;
            }
            strcpy(statusStr, g_DetectedNR);
            break;

          case IPMI_UNC_LOW:
          case IPMI_LNC_HIGH:
            if (NULL != pSeverity)
            {
              *pSeverity = CSS_SEVERITY_NORMAL;
            }
            strcpy(statusStr, g_Normal);
            break;
        }
      }
      else
      {
        /* set default string and state */
        if (NULL != pSeverity)
        {
          *pSeverity = CSS_SEVERITY_NORMAL;
        }
        strcpy(statusStr, g_Normal);

        /* now overide depending on the data */
        switch (eventData1)
        {
          case IPMI_UC_HIGH:
            /* These are de-asserts of an earlier condition we need to chk
             the threshold masks to see if this one has warning thresholds*/
            if (0 != (thrReadMask & IPMI_UNC_THRESHOLD_MASK))
            {
              if (NULL != pSeverity)
              {
                *pSeverity =  CSS_SEVERITY_WARNING;
              }
              strcpy(statusStr, g_Warning);
            }
            break;

          case IPMI_LC_LOW:
            /*These are de-asserts of an earlier condition we need to chk
             the threshold masks to see if this one has warning thresholds*/
            if (0 != (thrReadMask & IPMI_LNC_THRESHOLD_MASK))
            {
              if (NULL != pSeverity)
              {
                *pSeverity =  CSS_SEVERITY_WARNING;
              }
              strcpy(statusStr, g_Warning);
            }
            break;

          case IPMI_LNC_LOW:
          case IPMI_UNC_HIGH:
            if (NULL != pSeverity)
            {
              *pSeverity = CSS_SEVERITY_NORMAL;
            }
            strcpy(statusStr, g_Normal);
            break;
        }
      }
    }
    else
    {
      for (index = 0; index < g_GenericTableSize; index++)
      {
        /* Match the reading type */
        if (g_GenericTable[index].readingType == readingType)
        {
          /* correct index? */
          if (eventData1 < g_GenericTable[index].maxIndex)
          {
            strcpy(
                  statusStr, 
                  g_GenericTable[index].pSensorState[eventData1].state);

            /* unique case */
            if (sensorType == IPMI_SENSOR_MEMORY)
            {
              GetMemoryDIMMStr(memStr, data1, eventData2, eventData3);
              strcat(statusStr, memStr);
            }
            /* check for deassertion/assertion */
            if (FALSE == deasserted) /* asserted */
            {
              if (NULL != pSeverity)
              {
                *pSeverity = 
                g_GenericTable[index].pSensorState[eventData1].assertSeverity;
              }
              strcat(statusStr, g_wasAsserted);
            }
            else
            {
              if (NULL != pSeverity) /* deasserted */
              {
                *pSeverity = 
                  g_GenericTable[index].pSensorState[eventData1].deassertSeverity;                        
              }
              strcat(statusStr, g_wasDeasserted); 
            }
            break;
          }
        }
      }
    }
  }
  /* Sensor Specific reading type */
  else if (readingType == IPMI_READING_TYPE_SPEC_OFFSET)
  {
    for (index = 0; index < g_SpecificTableSize; index++)
    {
      /* Match the reading type */
      if (g_SpecificTable[index].readingType == sensorType)
      {
        /* Unique cases */ 
        if (sensorType == IPMI_SENSOR_POST_ERROR)
        {
          if (eventData1 == 0x0f)
          {
            if (NULL != pSeverity)
            {
              *pSeverity = CSS_SEVERITY_CRITICAL;
            }
            /* default to general POST error*/
            strcpy(statusStr, "POST fatal error");
            /*Check for extended POST message*/
            for (count = 0; count < g_PostMessagesSize; count++)
            {
              if (eventData2 == g_PostMessages[count].code)
              {
                strcpy(statusStr, g_PostMessages[count].message);
                break;
              }
            }
          }
          break;
        }

        /* correct index? */
        if (eventData1 < g_SpecificTable[index].maxIndex)
        {
          if (sensorType == IPMI_SENSOR_SLOT)
          {
            switch (eventData2 & 0x0f)
            {
              case 0:
                tmpStr = g_PciSlot;
                break;

              case 1:
                tmpStr = g_Drive;
                break;

              default:
                tmpStr = g_Slot;
                break;
            }
            strcpy(
                  statusStr,
                  tmpStr);
            strcat(
                  statusStr, 
                  g_SpecificTable[index].pSensorState[eventData1].state);
          }
          else
          {
            strcpy(
                  statusStr, 
                  g_SpecificTable[index].pSensorState[eventData1].state);
          }
          /* unique case */
          if ((sensorType == IPMI_SENSOR_PS) && ((data1&IPMI_OFFSET_MASK) == OFFSET_BYTE_6))
          {
            GetPowerSupplyWatts(memStr, data1, eventData2, eventData3);
            strcat(statusStr, memStr);
          }
          else if ((sensorType == IPMI_SENSOR_PS) && ((data1&IPMI_OFFSET_MASK) == OFFSET_BYTE_1))
          {
            GetPowerSupplyExtend(memStr, data1, eventData2, eventData3);
            strcat(statusStr, memStr);
          }
          else if (sensorType == IPMI_SENSOR_MEMORY)
          {
            GetMemoryDIMMStr(memStr, data1, eventData2, eventData3);
            strcat(statusStr, memStr);
          }
          else if ((sensorType == IPMI_SENSOR_CRIT_EVENT) ||
                   (sensorType == IPMI_SENSOR_NON_FATAL) ||
                   (sensorType == IPMI_SENSOR_IO_FATAL))
          {
            /* Decode OEM data */
            GetPCIInfoStr(memStr, data1, eventData2, eventData3);
            strcat(statusStr, memStr);
          }
          else if (sensorType == IPMI_SENSOR_VERSION_CHANGE)
          {
            GetExtentedHWConfigStr(memStr, data1, eventData2, eventData3);
            strcat(statusStr, memStr);
          }
          else if (sensorType == IPMI_SENSOR_WDOG2)
          {
            GetExtentedWD2Str(memStr, data1, eventData2);
            strcat(statusStr, memStr);
          }
          else if (sensorType == IPMI_SENSOR_LINK_TUNING)
          {
            GetExtentLinkTuningStr(memStr, data1, eventData2, eventData3);
            strcat(statusStr, memStr);
          }
          /* check for deassertion/assertion */
          if (FALSE == deasserted) /* asserted */
          {
            if (NULL != pSeverity)
            {
              *pSeverity = 
                g_SpecificTable[index].pSensorState[eventData1].assertSeverity;
            }
            strcat(
                  statusStr, 
                  g_wasAsserted);
            /* Unique cases */
            if (sensorType == IPMI_SENSOR_INTRUSION)
            {
              if ((data1 & OEM_BYTE_2_MASK) == OEM_BYTE_2)
              {
                if (eventData2 == 0x01)
                {
                  strcat(statusStr, g_whileOn); 
                }
                else if (eventData2 == 0x02)
                {
                  strcat(statusStr, g_whileOff); 
                }
              }
            }
          }
          else
          {
            if (NULL != pSeverity)
            {
              *pSeverity = 
                g_SpecificTable[index].pSensorState[eventData1].deassertSeverity;
            }
            /* Unique cases */
            if ((IPMI_SENSOR_DRIVE_SLOT == sensorType) &&
                (0 == eventData1))
            {
              strcpy(statusStr, "drive removed");
            }
            else
            {
              strcat(statusStr, g_wasDeasserted);
              /* Unique cases */
              if (sensorType == IPMI_SENSOR_INTRUSION)
              {
                if ((data1 & OEM_BYTE_2_MASK) == OEM_BYTE_2)
                {
                  if (eventData2 == 0x01)
                  {
                    strcat(statusStr, g_whileOn); 
                  }
                  else if (eventData2 == 0x02)
                  {
                    strcat(statusStr, g_whileOff); 
                  }
                }
              }
            }
          }
          break;
        }
      }
    }
  }
  /* OEM */
  else if ((readingType >= MIN_OEM_READING_TYPES) && 
           (readingType <= (unsigned char)MAX_OEM_READING_TYPES))
  {
    if (readingType == OEM_DELL_READING_TYPE)
    {
      if (NULL != pSeverity)
      {
        *pSeverity = CSS_SEVERITY_NORMAL;
      }
      strcpy(statusStr, g_oemEvent);
      if (FALSE == deasserted)
      {
        strcat(statusStr, g_wasAsserted);
      }
      else
      {
        strcat(statusStr, g_wasDeasserted);                    
      }
    }
  }
}


/****************************************************************************
    CSSFindSDRRecord
    This function searches for the SDR of a SEL entry
 ****************************************************************************/
static SDRType FAR *SelFindSDRRecord(
  _IN const void* pIPMISelEntry,
  _IN void*       puserParameter)
{
  unsigned char   ownerID;
  unsigned char   sensorNum;
  unsigned char   shareCount;
  unsigned char   tempOID;
  unsigned char   recType;
  unsigned char   tmpSensorNum;
  SDRType FAR *   pSdrHandle = NULL;
  IPMISELEntry*   pSelEntry  = (IPMISELEntry*)pIPMISelEntry;

  if ((NULL != USERAPIList.GetFirstSDR) &&
      (NULL != USERAPIList.GetNextSDR))
  {
    ownerID   = pSelEntry->generatorID1;
    sensorNum = pSelEntry->sensorNum;

    pSdrHandle = USERAPIList.GetFirstSDR(puserParameter);

    while (NULL != pSdrHandle)
    {
      recType = CSSSDRGetAttribute(
        pSdrHandle, 
        SDR_ATTRIBUTE_RECORD_TYPE,
        USERAPIList.Oem2IPMISDR);

      if ((SDR_COMPACT_SENSOR == recType) || 
          (SDR_FULL_SENSOR == recType))
      {
        shareCount = 1;
        if (SDR_COMPACT_SENSOR == recType)
        {
          shareCount = CSSSDRGetAttribute(
            pSdrHandle,
            SDR_ATTRIBUTE_SHARE_COUNT,
            USERAPIList.Oem2IPMISDR);
        }

        tempOID = CSSSDRGetAttribute(
          pSdrHandle,
          SDR_ATTRIBUTE_SENSOR_OWNER_ID,
          USERAPIList.Oem2IPMISDR);

        /*Find the SDR that matches the owner ID */
        if (tempOID == ownerID)
        {
          tmpSensorNum = CSSSDRGetAttribute(
            pSdrHandle,
            SDR_ATTRIBUTE_SENSOR_NUMBER,
            USERAPIList.Oem2IPMISDR);

          if ((tmpSensorNum <= sensorNum) && 
              (sensorNum < (tmpSensorNum + shareCount)))
          {
            /*found the record break */
            break;
          }
        }
      }
      pSdrHandle = USERAPIList.GetNextSDR(pSdrHandle, puserParameter);
    }
  }
  return pSdrHandle;
}


/****************************************************************************
    GetMemoryDIMMStr
    This function added DIMM number or memory board numbers for memory errors
 ****************************************************************************/
static void GetMemoryDIMMStr(
  _INOUT char *memStr,
  _IN unsigned char data1,
  _IN unsigned char data2, 
  _IN unsigned char data3)
{
  unsigned char node;
  unsigned char incr = 0;
  unsigned char i    = 0;
  unsigned char num;
  unsigned char dimmNum;
  unsigned char dimmsPerNode;
  char dimmStr[16];
  char cardStr[16];
  char bankStr[16];

  memStr[0]  = 0;
  dimmStr[0] = 0;
  cardStr[0] = 0;
  bankStr[0] = 0;

  if (OEM_ID_DELL == *((unsigned short*)&CSLFMethod.manufactureID[0]))
  {
    if ((data1 & OEM_BYTE_2_MASK) == OEM_BYTE_2)
    {
      /* Upper nible contains the card data, 0xf if card not present.
         This is the same for IPMI 1.5 and 2.0 */
      if (((data2 >> 4) != 0x0f) && ((data2 >> 4) < 0x08))
      {
        strcpy(memStr, " ( Memory Board ");
        cardStr[0] = (char)('A' + (char)(data2 >> 4));
        cardStr[1] = 0;
        strcat(memStr, cardStr);
      }
      if (0x0F != (data2 & 0x0f))
      {
        if (IPMI_1_5 == CSLFMethod.ipmiVersion)
        {
          /* Lower nibble contains BANK number */
          strcpy(bankStr, " Bank ");
          /* Bank is a zero based value */
          num = CSSlongToAscii((long)((data2 & 0x0f)+1), &bankStr[6], 10,0);
          bankStr[6 + num] = 0;
        }
        else
        {
          incr = ((data2 & 0x0f) << 3);
        }
      }
    }

    if ((data1 & OEM_BYTE_3_MASK) == OEM_BYTE_3)
    {
      /* if memStr byte zero is empty data2 does not 
         contain the  memory board string */
      if (memStr[0] == 0)
      {
        strcpy(memStr, " (");
      }

      /* For IPMI 1.5 need to copy Bank number and DIMM number into memStr */
      if (IPMI_1_5 == CSLFMethod.ipmiVersion)
      {
        strcat(memStr, bankStr);
        strcpy(dimmStr," DIMM ");
        dimmStr[6] = (char)('A' + data3);
        dimmStr[7] = 0;
        strcat(memStr, dimmStr);
      }
      else if (((data2 >> 4) > 0x07) && ((data2 >> 4) != 0x0F))   /* NUMA format */
      {
        /* default to 4 for data2[7,4] = 0x8 */
        dimmsPerNode = 4;
        if ((data2 >> 4) == 0x09)
        {
          dimmsPerNode = 6;
        }
        else if ((data2 >> 4) == 0x0A)
        {
          dimmsPerNode = 8;
        }
        else if ((data2 >> 4) == 0x0B)
        {
          dimmsPerNode = 9;
        }
        strcpy(dimmStr," DIMM_");
        for (i = 0; i < 8; i++)
        {
          if (BIT(i) & data3)
          {
            node = (incr + i)/dimmsPerNode;
            dimmNum = ((incr + i)%dimmsPerNode)+1;
            dimmStr[6] = node + 'A';
            num = CSSlongToAscii((long)dimmNum, &dimmStr[7], 10,0);
            dimmStr[7 + num] = 0;
            strcat(memStr, dimmStr);
          }
        }
      }
      else
      {
        strcpy(dimmStr," DIMM");

        for (i = 0; i < 8; i++)
        {
          if (BIT(i) & data3)
          {
            num = CSSlongToAscii((long)(i + incr + 1), &dimmStr[5], 10, 0);
            dimmStr[5 + num] = 0;
            strcat(memStr, dimmStr);
          }
        }
      }
    }

    /* Add closing ) if memstr is not empty */
    if (memStr[0] != 0)
    {
      strcat(memStr," )");
    }
  }
}

/****************************************************************************
    GetPCIInfoStr
    This function added PCI Slot or Bus/Device/Function number for PCI 
 ****************************************************************************/
static void GetPCIInfoStr(
  _INOUT char *memStr,
  _IN unsigned char data1,
  _IN unsigned char data2,
  _IN unsigned char data3)
{
  char *pu8Ptr;

  memStr[0] = 0;

  if (OEM_ID_DELL == *((unsigned short*)&CSLFMethod.manufactureID[0]))
  {
    /* Check for OEM data. Both data bytes are reqired to decode correctly */
    if ((data1 & OEM_BYTES_23_MASK) == OEM_BYTES_23)
    {
      pu8Ptr = memStr;

      if (data3 & PCI_IS_SLOT_MASK)
      {
        /* Data is slot number */
        strcpy(pu8Ptr, " (Slot ");
        pu8Ptr += 7;
        pu8Ptr += CSSlongToAscii((long)(data3 & PCI_SLOT_BUS_NUM_MASK), pu8Ptr, 10,0);
        strcpy(pu8Ptr, ")");
        pu8Ptr += 1;
      }
      else
      {
        /* Data is bus number, Device Number, Function Number */
        strcpy(pu8Ptr, " (Bus ");
        pu8Ptr += 6;
        pu8Ptr += CSSlongToAscii((long)(data3 & PCI_SLOT_BUS_NUM_MASK), pu8Ptr, 10,0);
        strcpy(pu8Ptr, " Device ");
        pu8Ptr += 8;
        pu8Ptr += CSSlongToAscii((long)((data2 & PCI_DEVICE_NUM_MASK) >> 3), pu8Ptr, 10,0);
        strcpy(pu8Ptr, " Function ");
        pu8Ptr += 10;
        pu8Ptr += CSSlongToAscii((long)(data2 & PCI_FUNCTION_NUM_MASK), pu8Ptr, 10,0);
        strcpy(pu8Ptr, ")");
        pu8Ptr += 1;
      }
      *pu8Ptr = 0;    /* null terminate the last byte */
    }
  }
}

/****************************************************************************
    GetExtentedHWConfigStr

 ****************************************************************************/
static void GetExtentedHWConfigStr(
  _INOUT char *configStr,
  _IN unsigned char  data1,
  _IN unsigned char  data2, 
  _IN unsigned char  data3)
{
  unsigned char data;
  unsigned char num;
  char number[32];
  configStr[0] = 0;

  if (IPMI_2_0 == CSLFMethod.ipmiVersion)
  {
    if ((data1&IPMI_OFFSET_MASK) == 3)
    {
      if ((data1 & OEM_BYTES_23_MASK) == OEM_BYTES_23)
      {
        data = data2>>5;
        switch(data)
        {
          case 0:
            strcat(configStr," (BIOS ");
            if ((data2&0x1F) != 0x1F)
            {
              number[0] = 0;
              CSSlongToAscii((long)(data2&0x1F), number, 10, 0);
              strcat(configStr, number);
            }
            break;
          case 1:
            strcat(configStr," (BMC ");
            if ((data2&0x1F) != 0x1F)
            {
              number[0] = 0;
              CSSlongToAscii((long)(data2&0x1F), number, 10, 0);
              strcat(configStr, number);
            }
            break;
          case 2:
            strcat(configStr," (iDRAC ");
            if ((data2&0x1F) != 0x1F)
            {
              number[0] = 0;
              CSSlongToAscii((long)(data2&0x1F), number, 10, 0);
              strcat(configStr, number);
            }
            break;
          case 3:
            strcat(configStr," (CMC ");
            if ((data2&0x1F) != 0x1F)
            {
              number[0] = 0;
              CSSlongToAscii((long)(data2&0x1F), number, 10, 0);
              strcat(configStr, number);
            }
            break;
          case 4:
            strcat(configStr," (NIC ");
            if ((data2&0x1F) != 0x1F)
            {
              number[0] = 0;
              CSSlongToAscii((long)(data2&0x1F), number, 10, 0);
              strcat(configStr, number);
            }
            break;
          default:
            /* not a supported fw device */
            configStr[0] = 0;
            break;
        }
        data = data3>>5;
        if (configStr[0] != 0)
        {
          strcat(configStr," with ");
          switch(data)
          {
            case 0:
              strcat(configStr,"BIOS ");
              if ((data3&0x1F) != 0x1F)
              {
                number[0] = 0;
                CSSlongToAscii((long)(data3&0x1F), number, 10, 0);
                strcat(configStr, number);
              }
              break;
            case 1:
              strcat(configStr,"BMC ");
              if ((data3&0x1F) != 0x1F)
              {
                number[0] = 0;
                CSSlongToAscii((long)(data3&0x1F), number, 10, 0);
                strcat(configStr, number);
              }
              break;
            case 2:
              strcat(configStr,"iDRAC ");
              if ((data3&0x1F) != 0x1F)
              {
                number[0] = 0;
                CSSlongToAscii((long)(data3&0x1F), number, 10, 0);
                strcat(configStr, number);
              }
              break;
            case 3:
              strcat(configStr,"CMC ");
              if ((data3&0x1F) != 0x1F)
              {
                number[0] = 0;
                CSSlongToAscii((long)(data3&0x1F), number, 10, 0);
                strcat(configStr, number);
              }
              break;
            case 4:
              strcat(configStr,"NIC ");
              if ((data3&0x1F) != 0x1F)
              {
                number[0] = 0;
                CSSlongToAscii((long)(data3&0x1F), number, 10, 0);
                strcat(configStr, number);
              }
              break;
            default:
              /* not a supported fw device */
              configStr[0] = 0;
              break;
          }
          if (configStr[0] != 0)
          {
            strcat(configStr,")");
          }
        }
      }
    }
    else if (((data1&0x0f) == 6) && ((data1 & OEM_BYTE_3_MASK) == OEM_BYTE_3))
    {
      if ((data3&0x80) == 0x80)
      {
        strcpy(configStr," device slot ");
        num = CSSlongToAscii((long)(data3&0x7F), &configStr[13], 10,0);
        configStr[13 + num] = 0;
      }
    }
    /* both data bytes must be enable */
    else if ((data1 & OEM_BYTES_23_MASK) == OEM_BYTES_23)
    {
      /* if data byte 2 is not equal to 2 then do not decode */
      if ((data2 & 0x0f) == 2)
      {
        strcpy(configStr," (BMC Firmware ");
        switch (data3 & 0x0f)
        {
          case 1:
            strcat(configStr, "and CPU mismatch");
            break;
          case 0:
            strcat(configStr, "and other hardware mismatch");
            break;
        }
        strcat(configStr,")");
      }
    }
  }
}

/****************************************************************************
    GetExtentedWD2Str

 ****************************************************************************/
static void GetExtentedWD2Str(
  _INOUT char* memStr,
  _IN unsigned char data1,
  _IN unsigned char data2)
{
  memStr[0] = 0;
  /* upper nibble equals 0xC0 or 0x80 or 0x40 */
  if (data1 & OEM_BYTE_2_MASK) 
  {
    if (data2 & 0x04)
    {
      strcpy(memStr," by SMS/OS timer");
    }
  }                             
}

/****************************************************************************
    GetExtentLinkTuningStr

 ****************************************************************************/
static void GetExtentLinkTuningStr(
  _INOUT char*      memStr, 
  _IN unsigned char data1, 
  _IN unsigned char data2,
  _IN unsigned char data3)
{
  char *pu8Ptr;
  unsigned char slot;
  unsigned char slotsPerNode = 1;

  memStr[0] = 0;

  /* Check for OEM data. Both data bytes are reqired to decode correctly */
  if ((data1 & OEM_BYTES_23_MASK) == OEM_BYTES_23)
  {
    if ((data1 & 0x0f) == 1)
    {
      pu8Ptr = memStr;
      /* Data is bus number, Device Number, Function Number */
      strcpy(pu8Ptr, " (Bus ");
      pu8Ptr += 6;
      pu8Ptr += CSSlongToAscii((long)(data3 & PCI_SLOT_BUS_NUM_MASK), pu8Ptr, 10, 0);
      strcpy(pu8Ptr, " Device ");
      pu8Ptr += 8;
      pu8Ptr += CSSlongToAscii((long)((data2 & PCI_DEVICE_NUM_MASK) >> 3), pu8Ptr, 10, 0);
      strcpy(pu8Ptr, " Function ");
      pu8Ptr += 10;
      pu8Ptr += CSSlongToAscii((long)(data2 & PCI_FUNCTION_NUM_MASK), pu8Ptr, 10, 0);
      strcpy(pu8Ptr, ")");
      pu8Ptr += 1;
      *pu8Ptr = 0;    /* null terminate the last byte */
    }
    else if ((data1 & IPMI_OFFSET_MASK) == 2)
    {
      if (data3 & PCI_IS_SLOT_MASK)
      {
        /* default to 1 per node */
        /* Single Height blade format */
        slotsPerNode = 1;
        /* Double height blade format */
        if ((data2>>6) == 1)
        {
          slotsPerNode = 2;
        }
        /* Double height double width blade format */
        else if ((data2>>6) == 2)
        {
          slotsPerNode = 4;
        }
        slot = ((data3&0x7F)-1); /* make slotzero base */
        strcpy(memStr, " (Mezz ");
        /* odd number is B, even number is C */
        if (data3&0x01)
        {
           memStr[7] = (char)'B';
        }
        else
        {
          memStr[7] = (char)'C';
        }
        if (slotsPerNode < 2) /* no number appended */
        {
          memStr[8] = ')';
          memStr[9] = 0;
        }
        else
        {
          memStr[8] = (char)('0' + ((slot/slotsPerNode)+1));
          memStr[9] = ')';
          memStr[10] = 0;
        }
      }
      else
      {
        strcpy(memStr, " (embedded)");
      }
    }
  }
}

static void GetPowerSupplyWatts(
  _INOUT char*      memStr, 
  _IN unsigned char data1, 
  _IN unsigned char eventData2,
  _IN unsigned char eventData3)
{
  long wattage = 0;
  char wattageStr[32];

  /* Check for OEM data. Both data bytes are reqired to decode correctly */
  if (((data1 & OEM_BYTES_23_MASK) == OEM_BYTES_23) && ((data1&IPMI_OFFSET_MASK) == OFFSET_BYTE_6))
  {
    /* check for error type in byte 3 */
    if ((eventData3&0x0F) == 0x03)
    {
      memStr[0] = 0;
      strcpy(memStr, ":Power Supply Mismatch (Power Supply ");
      wattageStr[0] = 0;
      wattage = ((long)eventData2<<4) + (eventData3>>4); 
      CSSlongToAscii(wattage, wattageStr, 10, 0);
      if (wattageStr[0] != 0)
      {
        strcat(memStr, wattageStr);
        strcat(memStr, " Watts)");
      }
    }
  }
}

static void GetPowerSupplyExtend(
  _INOUT char*      memStr, 
  _IN unsigned char data1, 
  _IN unsigned char eventData2,
  _IN unsigned char eventData3)
{
  UNREFERENCED_PARAMETER(eventData3);
  /* Check for OEM data in byte 2 */
  if (((data1 & OEM_BYTES_23_MASK) == OEM_BYTE_2) && ((data1&IPMI_OFFSET_MASK) == OFFSET_BYTE_1))
  {
    /* check for error type in byte 3 */
    if (eventData2 == 0x01)
    {
      memStr[0] = 0;
      strcpy(memStr, " (PMBus communication error)");
    }
  }
}

/* Code End */

