/*
	Copyright (c) 2007 Dell Inc
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include <ctype.h>
#include <limits.h>						/* _MIN and _MAX*/
#include <ipmitool/ipmi.h>
#include <ipmitool/ipmi_intf.h>
#include <ipmitool/helper.h>
#include <ipmitool/log.h>
#include <ipmitool/ipmi_sel.h>
#include <ipmitool/ipmi_delloem.h>
#include <ipmitool/ipmi_fru.h>
#include <ipmitool/ipmi_sdr.h>
#include <ipmitool/ipmi_mc.h>
#include <ipmitool/ipmi_sensor.h>
#include <ipmitool/ipmi_sel.h>
#include <ipmitool/bswap.h>
#include <ipmitool/ipmi_sdr.h>
#include <ipmitool/ipmi_entity.h>
#include <ipmitool/ipmi_fru.h>
#include <ipmitool/ipmi_sensor.h>
/*------------ipmi headers------------------*/
#include <ipmitool/cssstat.h>
#include <ipmitool/cssipmi.h>
#include <ipmitool/cssapi.h>
#include <ipmitool/csssupt.h>
#include <ipmitool/cssdprod.h>
#include <ipmitool/cmnslf.h>

#define dprint(x) printf("\n\r"#x" = %d",x);
#define dprinth(x) printf("\n\r"#x" = 0x%h",x);


/*--------------time header-----------------*/
#include <time.h>


extern int read_fru_area(struct ipmi_intf * intf, struct fru_info *fru, 
		uint8_t id,uint32_t offset, uint32_t length, uint8_t *frubuf);
extern char *get_fru_area_str(uint8_t * data, uint32_t * offset);
uint16_t inputwattage=0;//for debuging 
static void
ipmi_sysinfo_print_board(struct ipmi_intf * intf, struct fru_info * fru,
		     uint8_t id, uint32_t offset, uint8_t ipmi_version);
static int ipmi_sysinfo_get_tag(struct ipmi_intf * intf, char* tagstring, uint8_t tag_type, uint8_t length);
extern int  ipmi_sel_get(struct ipmi_intf * intf, uint16_t id);
static int
ipmi_sysinfo_fru(struct ipmi_intf * intf, uint8_t id, uint8_t ipmi_version);
static char*  ipmi_get_bib_info(uint8_t * fru_data, uint8_t bib_type);
uint16_t ipmi_sel_get_record(struct ipmi_intf * intf, uint16_t curr_id, uint8_t* sel_entry);
int
ipmi_sdr_reserve(struct ipmi_intf *intf, uint16_t * reserve_id);
int ipmi_sdr_get(struct ipmi_intf *intf, uint16_t reserveId, uint8_t* datapt, uint16_t recordId);
SDRType * ipmi_sdr_get_first(void* x);
SDRType * ipmi_sdr_get_next(SDRType *w, void *x);
int ipmi_sel_sysinfo(struct ipmi_intf *intf, DECODING_METHOD *method);
int ipmi_sel_print(struct ipmi_intf * intf, uint16_t count, int printOption);
static uint16_t
ipmi_sel_reserve(struct ipmi_intf * intf);
static int ipmi_sensor_print(SDRType* pSdr, int* pSensorIndex, struct ipmi_intf * intf);
int
ipmi_sensor_get_sensor_reading(unsigned char sensorOwner, unsigned char sensorNumber,
							   IPMISensorReadingType* sensorReadingData,
							   void* x);
SDRType* ipmi_sdr_get_test(struct ipmi_intf *intf, uint16_t reserveId, uint8_t* datapt, uint16_t recordId);
SDRType* ipmi_sdr_get_test2(struct ipmi_intf *intf, uint16_t reserveId, uint8_t* datapt, uint16_t recordId);
SDRType* ipmi_sdr_get_test3(struct ipmi_intf *intf, uint16_t reserveId, uint8_t* datapt, uint16_t recordId);
static int ipmi_powermgmt(struct ipmi_intf* intf);
static void ipmi_time_to_str(time_t rawTime, char* strTime);
static char * ipmi_sel_timestamp(uint32_t stamp);
static int make_int(const char *str, int *value);
static int ipmi_build_sdr_first(struct ipmi_intf *intf, uint8_t *data);
int ipmi_build_sdr_next(struct ipmi_intf *intf, uint8_t *sdr_in, uint8_t *data);
static int ipmi_sdr_build_table(struct ipmi_intf *intf); 
/*om6.0*/
static int ipmi_lcd_get_status_val(struct ipmi_intf * intf, LCD_STATUS* lcdstatus);
static int ipmi_print_power_supply_info(IPMISDR*  pRecord,int* pSensorIndex, struct ipmi_intf * intf,IPMI_POWER_SUPPLY_INFO* powersupplyinfo);

//static int ipmi_get_power_supply_info(struct ipmi_intf* intf, int entity_id , int entity_instance ,IPMI_POWER_SUPPLY_INFO* powersupplyinfo);
static int ipmi_get_power_supply_info(struct ipmi_intf* intf,IPMISDR*  pRecord ,IPMI_POWER_SUPPLY_INFO* powersupplyinfo);
static int ipmi_get_power_supply_rating(struct ipmi_intf* intf);

void ipmi_lcd_status_print( LCD_STATUS* lcdstatus);
static int ipmi_lcd_get_status(struct ipmi_intf * intf );
static int ipmi_lcd_set_kvm(struct ipmi_intf * intf, char status);
static int ipmi_get_power_consumption_data(struct ipmi_intf* intf,uint8_t unit);

static int ipmi_get_instan_power_consmpt_data(struct ipmi_intf* intf,
			IPMI_INST_POWER_CONSUMPTION_DATA* instpowerconsumptiondata);
static int ipmi_get_peak_power_consmpt_data(struct ipmi_intf* intf,uint32_t* pPeakpower);
uint64_t watt_to_btuphr_conversion(uint32_t powerinwatt);
uint32_t btuphr_to_watt_conversion(uint64_t powerinbtuphr);
static void ipmi_print_get_peak_power_consmpt_data(uint32_t* pPeakpower,uint8_t  unit,uint16_t inputwattage);
static void ipmi_print_get_instan_power_consmpt_data(IPMI_INST_POWER_CONSUMPTION_DATA* instpowerconsumptiondata,
				uint8_t  unit,uint16_t inputwattage);
static int ipmi_print_get_power_consmpt_data(struct ipmi_intf* intf,uint8_t  unit);
static int ipmi_get_avgpower_consmpt_history(struct ipmi_intf* intf,IPMI_AVGPOWER_CONSUMP_HISTORY* avgpower);

//static int ipmi_get_peakpower_consmpt_history(struct ipmi_intf* intf);
static int ipmi_get_peakpower_consmpt_history(struct ipmi_intf* intf,IPMI_PEAKPOWER_CONSUMP_HISTORY * pstPeakpower);

static int ipmi_print_power_consmpt_history(struct ipmi_intf* intf,int unit );
static int ipmi_set_power_cap(struct ipmi_intf* intf,int unit,int val );
static int ipmi_print_power_cap(struct ipmi_intf* intf,uint8_t unit );
static int ipmi_get_power_cap(struct ipmi_intf* intf,IPMI_POWER_CAP* ipmipowercap );
uint8_t  getpowersupplyfruinfo(struct ipmi_intf *intf, uint8_t id, struct fru_header header,
	struct fru_info fru);

/*om6.0*/
SDRType* sdr_get_first();
SDRType* sdr_get_next(SDRType* pPreviousSDR);


POWER_HEADROOM powerheadroom;

LCD_MODE lcd_mode;

uint8_t iDRAC_FLAG=0;
uint8_t PowercapSetable_flag=0;
uint8_t PowercapstatusFlag=0;

uint8_t  data[64];

static const struct valstr threshold_vals[] = {
	{UPPER_NON_RECOV_SPECIFIED, "Upper Non-Recoverable"},
	{UPPER_CRIT_SPECIFIED, "Upper Critical"},
	{UPPER_NON_CRIT_SPECIFIED, "Upper Non-Critical"},
	{LOWER_NON_RECOV_SPECIFIED, "Lower Non-Recoverable"},
	{LOWER_CRIT_SPECIFIED, "Lower Critical"},
	{LOWER_NON_CRIT_SPECIFIED, "Lower Non-Critical"},
	{0x00, NULL},
};

typedef struct _SDRList
{
	uint8_t data[MAX_SDR_SIZE];
	uint8_t bIsLast;
} SDRList;


SDRList SDR[MAX_SDR_ENTRIES];
uint16_t currentSDR;
uint16_t totalSDRCount;


/* Dell OEM LCD command*/
/* returns the current lcd configuration
 * 0 = User defined
 * 1 = Default
 * 2 = None
 */
int
ipmi_lcd_get_configure_command_wh (struct ipmi_intf * intf)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];

	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_GET_SYS_INFO;
	req.msg.data_len = 4;
	req.msg.data = data;
	data[0] = 0;
	data[1] = IPMI_DELL_LCD_CONFIG_SELECTOR;
	data[2] = 0;
	data[3] = 0;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		lprintf(LOG_ERR, " Error getting LCD configuration");
		return -1;
	}else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb)){
	
            lprintf(LOG_ERR, " Error getting LCD configuration: Command not supported on this system."); 

       } else if (rsp->ccode > 0) {
		lprintf(LOG_ERR, " Error getting LCD configuration: %s",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}

    lcd_mode= *((LCD_MODE*)(rsp->data));
	return 0;
}


// Dell OEM LCD command
int
ipmi_lcd_get_configure_command (struct ipmi_intf * intf, uint8_t *command)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];

	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_GET_SYS_INFO;
	req.msg.data_len = 4;
	req.msg.data = data;
	data[0] = 0;
	data[1] = IPMI_DELL_LCD_CONFIG_SELECTOR;
	data[2] = 0;
	data[3] = 0;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL)
	{
		lprintf(LOG_ERR, " Error getting LCD configuration");
		return -1;
	}
	else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb))
	{
	    lprintf(LOG_ERR, " Error getting LCD configuration: Command not supported on this system."); 
		return -1;
    }
	else if (rsp->ccode > 0) 
	{
		lprintf(LOG_ERR, " Error getting LCD configuration: %s",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}

	// rsp->data[0] is the rev
	*command = rsp->data[1];

	return 0;
}


int
ipmi_lcd_set_configure_command (struct ipmi_intf * intf, int command)
{
#define LSCC_DATA_LEN 2

	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[2];

	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_SET_SYS_INFO;
	req.msg.data_len = 2;
	req.msg.data = data;
	data[0] = IPMI_DELL_LCD_CONFIG_SELECTOR;
	data[1] = command;			// command - custom, default, none

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) 
	{
		lprintf(LOG_ERR, " Error setting LCD configuration");
		return -1;
	}
	else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb))
	{
        lprintf(LOG_ERR, " Error setting LCD configuration: Command not supported on this system."); 

    }
	else if (rsp->ccode > 0) 
	{
		lprintf(LOG_ERR, " Error setting LCD configuration: %s",
			val2str(rsp->ccode, completion_code_vals));

		return -1;
	}

	return 0;
}


/* sets the current lcd configuration
 * 0 = User defined
 * 1 = Default
 * 2 = None
 */
int
ipmi_lcd_set_configure_command_wh (struct ipmi_intf * intf, uint32_t  mode ,uint16_t lcdquallifier, uint8_t errordisp)
{
#define LSCC_DATA_LEN 2

	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[13];
  
	ipmi_lcd_get_configure_command_wh(intf);
	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_SET_SYS_INFO;
	req.msg.data_len = 13;
	req.msg.data = data;
	data[0] = IPMI_DELL_LCD_CONFIG_SELECTOR;
    
    if(mode!=0xFF)
    {
         
		data[1] = mode&0xFF;			/* command - custom, default, none*/
       	data[2]=(mode&0xFF00)>>8;
       	data[3]=(mode&0xFF0000)>>16;
       	data[4]=(mode&0xFF000000)>>24;
	}
    else
    {
    	data[1] = (lcd_mode.lcdmode)&0xFF;			/* command - custom, default, none*/
       	data[2]=((lcd_mode.lcdmode)&0xFF00)>>8;
       	data[3]=((lcd_mode.lcdmode)&0xFF0000)>>16;
       	data[4]=((lcd_mode.lcdmode)&0xFF000000)>>24;
	}
       
    if(lcdquallifier!=0xFF)
    {
    	if(lcdquallifier==0x01)
        {
			data[5] =(lcd_mode.lcdquallifier)|0x01;			/* command - custom, default, none*/

		}
        else  if(lcdquallifier==0x00)
		{
			data[5] =(lcd_mode.lcdquallifier)&0xFE;			/* command - custom, default, none*/
		}
        else if (lcdquallifier==0x03)
        {
			data[5] =(lcd_mode.lcdquallifier)|0x02;			/* command - custom, default, none*/
        }
        else if (lcdquallifier==0x02)
		{
			data[5] =(lcd_mode.lcdquallifier)&0xFD;	
		}
	}
    else
    {
    	data[5]=lcd_mode.lcdquallifier;
    }   
    if(errordisp!=0xFF)
    {
        data[11]=errordisp;
    }
    else
    {
        data[11]=lcd_mode.error_display;
    }
    rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) 
	{
		lprintf(LOG_ERR, " Error setting LCD configuration");
		return -1;
	}
	else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb))
	{
		lprintf(LOG_ERR, " Error setting LCD configuration: Command not supported on this system."); 
	}
	else if (rsp->ccode > 0) 
	{
		lprintf(LOG_ERR, " Error setting LCD configuration: %s",
			val2str(rsp->ccode, completion_code_vals));

		return -1;
	}

	return 0;
}


/* Used to get the platform model name, or any other parameter which stores
 * data in the same format
 */
static
ipmi_lcd_get_platform_model_name (struct ipmi_intf * intf, char* lcdstring, uint8_t max_length, uint8_t field_type)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];
	IPMI_DELL_LCD_STRING * lcdstringblock;
	int lcdstring_len = 0;
	int bytes_copied = 0;


	int ii;

	for (ii = 0; ii < 4; ii++)
	{
		int bytes_to_copy;

		req.msg.netfn = IPMI_NETFN_APP;
		req.msg.cmd = IPMI_GET_SYS_INFO;
		req.msg.data_len = 4;
		req.msg.data = data;
		data[0] = 0;				/* get parameter*/
		data[1] = field_type;
		data[2] = ii;
		data[3] = 0;


		rsp = intf->sendrecv(intf, &req);
		if (rsp == NULL) {
			lprintf(LOG_ERR, " Error getting platform model name");
		} else if (rsp->ccode > 0) {
			lprintf(LOG_ERR, " Error getting platform model name: %s",
				val2str(rsp->ccode, completion_code_vals));
		}

		lcdstringblock = (IPMI_DELL_LCD_STRING *) (void *) rsp->data;
		
		/* first block is different - 14 bytes*/
		if (0 == ii) {
			lcdstring_len = lcdstringblock->lcd_string.selector_0_string.length;

			if (lcdstring_len < 1 || lcdstring_len > max_length)
				break;

			bytes_to_copy = MIN(lcdstring_len, IPMI_DELL_LCD_STRING1_SIZE);
			memcpy (lcdstring, lcdstringblock->lcd_string.selector_0_string.data, bytes_to_copy);
		} else {
			int string_offset;

			bytes_to_copy = MIN(lcdstring_len - bytes_copied, IPMI_DELL_LCD_STRINGN_SIZE);
			if (bytes_to_copy < 1)
				break;
			string_offset = IPMI_DELL_LCD_STRING1_SIZE + IPMI_DELL_LCD_STRINGN_SIZE * (ii-1);
			memcpy (lcdstring+string_offset, lcdstringblock->lcd_string.selector_n_data, bytes_to_copy);
		}
		

		bytes_copied += bytes_to_copy;

		if (bytes_copied >= lcdstring_len)
            
			break;
	}

}

static int
ipmi_lcd_get_single_line_text (struct ipmi_intf * intf, char* lcdstring, uint8_t max_length)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];
	IPMI_DELL_LCD_STRING * lcdstringblock;
	int lcdstring_len = 0;
	int bytes_copied = 0;
	int ii;

	for (ii = 0; ii < 4; ii++) {
		int bytes_to_copy;

		req.msg.netfn = IPMI_NETFN_APP;
		req.msg.cmd = IPMI_GET_SYS_INFO;
		req.msg.data_len = 4;
		req.msg.data = data;
		data[0] = 0;				/* get parameter*/
		data[1] = IPMI_DELL_LCD_STRING_SELECTOR;
		data[2] = ii;				/* block selector*/
		data[3] = 00;				/* set selector (n/a)*/

		rsp = intf->sendrecv(intf, &req);
		if (rsp == NULL) {
			lprintf(LOG_ERR, " Error getting text data");
			return -1;
		} else if (rsp->ccode > 0) {
			lprintf(LOG_ERR, " Error getting text data: %s",
				val2str(rsp->ccode, completion_code_vals));
			return -1;
		}

		lcdstringblock = (IPMI_DELL_LCD_STRING *) (void *) rsp->data;
		
		/* first block is different - 14 bytes*/
		if (0 == ii)
		{
			lcdstring_len = lcdstringblock->lcd_string.selector_0_string.length;

			if (lcdstring_len < 1 || lcdstring_len > max_length)
				break;

			bytes_to_copy = MIN(lcdstring_len, IPMI_DELL_LCD_STRING1_SIZE);
			memcpy (lcdstring, lcdstringblock->lcd_string.selector_0_string.data, bytes_to_copy);
		}
		else
		{
			int string_offset;

			bytes_to_copy = MIN(lcdstring_len - bytes_copied, IPMI_DELL_LCD_STRINGN_SIZE);
			if (bytes_to_copy < 1)
				break;
			string_offset = IPMI_DELL_LCD_STRING1_SIZE + IPMI_DELL_LCD_STRINGN_SIZE * (ii-1);
			memcpy (lcdstring+string_offset, lcdstringblock->lcd_string.selector_n_data, bytes_to_copy);
		}

		bytes_copied += bytes_to_copy;
		if (bytes_copied >= lcdstring_len)
			break;
	}
	return 0;
}

static int
ipmi_lcd_get_info_wh(struct ipmi_intf * intf)

{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];
	uint8_t command = 0;
	IPMI_DELL_LCD_CAPS* lcd_caps;
	char lcdstring[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};
	int rc;

	
	printf("LCD info\n");
     
	if (ipmi_lcd_get_configure_command_wh (intf) != 0) 
	{
    	return -1;
	}
	else 
	{
		if (lcd_mode.lcdmode== IPMI_DELL_LCD_CONFIG_DEFAULT) 
		{
			char text[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};

			ipmi_lcd_get_platform_model_name(intf, text, 
						IPMI_DELL_LCD_STRING_LENGTH_MAX,
						IPMI_DELL_PLATFORM_MODEL_NAME_SELECTOR);

			if (text == NULL)
				return -1;
			printf("    Setting:Model name\n");
			printf("    Line 1:  %s\n", text);
		}
		else if (lcd_mode.lcdmode == IPMI_DELL_LCD_CONFIG_NONE) 
		{
			printf("    Setting:   none\n");
		}
		else if (lcd_mode.lcdmode == IPMI_DELL_LCD_CONFIG_USER_DEFINED) 
		{
			req.msg.netfn = IPMI_NETFN_APP;
			req.msg.cmd = IPMI_GET_SYS_INFO;
			req.msg.data_len = 4;
			req.msg.data = data;
			data[0] = 0;				/* get parameter*/
			data[1] = IPMI_DELL_LCD_GET_CAPS_SELECTOR;
			data[2] = 0;				/* set selector (n/a)*/
			data[3] = 0;				/* block selector (n/a)*/

			printf("    Setting: User defined\n");

			rsp = intf->sendrecv(intf, &req);
			if (rsp == NULL)
			{
				lprintf(LOG_ERR, " Error getting LCD capabilities.");
				return -1;
			}
			else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb))
			{
            	lprintf(LOG_ERR, " Error getting LCD capabilities: Command not supported on this system.");
            }
			else if (rsp->ccode > 0) 
			{
				lprintf(LOG_ERR, " Error getting LCD capabilities: %s",
					val2str(rsp->ccode, completion_code_vals));
				return -1;
			}

			lcd_caps = (IPMI_DELL_LCD_CAPS *)rsp->data;
			if (lcd_caps->number_lines > 0) 
			{
				memset(lcdstring, 0, IPMI_DELL_LCD_STRING_LENGTH_MAX+1);

				rc = ipmi_lcd_get_single_line_text (intf, lcdstring, lcd_caps->max_chars[0]);
				printf("    Text:    %s\n", lcdstring);
			}
			else 
			{
				printf("    No lines to show\n");
			}
		}
		else if (lcd_mode.lcdmode == IPMI_DELL_LCD_iDRAC_IPV4ADRESS) 
		{
			printf("    Setting:   IPV4 Address\n");
		}
        else if (lcd_mode.lcdmode == IPMI_DELL_LCD_IDRAC_MAC_ADDRESS) 
		{
			printf("    Setting:   MAC Address\n");
		}
        else if (lcd_mode.lcdmode == IPMI_DELL_LCD_OS_SYSTEM_NAME) 
		{
			printf("    Setting:   OS System Name\n");
		}
        else if (lcd_mode.lcdmode == IPMI_DELL_LCD_SERVICE_TAG) 
		{
			printf("    Setting:   System Tag\n");
		}
        else if (lcd_mode.lcdmode == IPMI_DELL_LCD_iDRAC_IPV6ADRESS) 
		{
			printf("    Setting:  IPV6 Address\n");
		}
        else if (lcd_mode.lcdmode == IPMI_DELL_LCD_AMBEINT_TEMP) 
		{
			printf("    Setting:  Ambient Temp\n");
            if(lcd_mode.lcdquallifier&0x02)
				printf("    Unit:  F\n");  
            else
                printf("    Unit:  C\n");  
		}
        else if (lcd_mode.lcdmode == IPMI_DELL_LCD_SYSTEM_WATTS)
		{
			printf("    Setting:  System Watts\n");
       
            if(lcd_mode.lcdquallifier&0x01)
            	printf("    Unit:  BTU/hr\n");  
            else
				printf("    Unit:  Watt\n"); 
                       
		}
        if(lcd_mode.error_display==IPMI_DELL_LCD_ERROR_DISP_SEL)
        	printf("    Error Display:  SEL\n");
        else if(lcd_mode.error_display==IPMI_DELL_LCD_ERROR_DISP_VERBOSE)
            printf("    Error Display:  Simple\n");
	}

	return 0;
}

static int ipmi_lcd_get_info(struct ipmi_intf * intf)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];
	IPMI_DELL_LCD_CAPS * lcd_caps;
	uint8_t command = 0;
	char lcdstring[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};
	int rc;	

	printf("LCD info\n");

	if (ipmi_lcd_get_configure_command (intf, &command) != 0)
	{
		return -1;
	}
	else
	{
		if (command == IPMI_DELL_LCD_CONFIG_DEFAULT)
		{
			memset (lcdstring,0,IPMI_DELL_LCD_STRING_LENGTH_MAX+1);

			ipmi_lcd_get_platform_model_name(intf, lcdstring, IPMI_DELL_LCD_STRING_LENGTH_MAX,
													IPMI_DELL_PLATFORM_MODEL_NAME_SELECTOR);

			printf("    Setting: default\n");
			printf("    Line 1:  %s\n", lcdstring);
		}
		else if (command == IPMI_DELL_LCD_CONFIG_NONE)
		{
			printf("    Setting:   none\n");
		}
		else if (command == IPMI_DELL_LCD_CONFIG_USER_DEFINED)
		{
			req.msg.netfn = IPMI_NETFN_APP;
			req.msg.cmd = IPMI_GET_SYS_INFO;
			req.msg.data_len = 4;
			req.msg.data = data;
			data[0] = 0;				// get parameter
			data[1] = IPMI_DELL_LCD_GET_CAPS_SELECTOR;
			data[2] = 0;				// set selector (n/a)
			data[3] = 0;				// block selector (n/a)

			printf("    Setting: custom\n");

			rsp = intf->sendrecv(intf, &req);
			if (rsp == NULL)
			{
				lprintf(LOG_ERR, " Error getting LCD capabilities.");
				return -1;
			}
			else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb))
			{
            	lprintf(LOG_ERR, " Error getting LCD capabilities: Command not supported on this system.");
            }
			else if (rsp->ccode > 0)
			{
				lprintf(LOG_ERR, " Error getting LCD capabilities: %s",
					val2str(rsp->ccode, completion_code_vals));
				return -1;
			}

			lcd_caps = (IPMI_DELL_LCD_CAPS *)(void *)rsp->data;
			if (lcd_caps->number_lines > 0)
			{
				memset (lcdstring,0,IPMI_DELL_LCD_STRING_LENGTH_MAX+1);
				rc = ipmi_lcd_get_single_line_text (intf, lcdstring, lcd_caps->max_chars[0]);
				printf("    Text:    %s\n", lcdstring);
			}
			else
			{
				printf("    No lines to show\n");
			}
		}
	}

	return 0;
}

static int
ipmi_lcd_get_status_val(struct ipmi_intf * intf, LCD_STATUS* lcdstatus)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];
	

		req.msg.netfn = IPMI_NETFN_APP;
		req.msg.cmd = IPMI_GET_SYS_INFO;
		req.msg.data_len = 4;
		req.msg.data = data;
		data[0] = 0;				// get parameter
		data[1] = IPMI_DELL_LCD_STATUS_SELECTOR;
		data[2] = 0;				// block selector
		data[3] = 0;		
        // set selector (n/a)
             // lprintf(LOG_ERR, "I am here 1");
		rsp = intf->sendrecv(intf, &req);
		if (rsp == NULL) 
		{
			lprintf(LOG_ERR, " Error getting LCD Status");
			return -1;
		}
		else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb)) 
		{
			lprintf(LOG_ERR, " Error getting LCD status: Command not supported on this system.");
			return -1;
		}
		else if (rsp->ccode > 0) 
		{
			lprintf(LOG_ERR, " Error getting LCD Status: %s",
				val2str(rsp->ccode, completion_code_vals));
			return -1;
		}
		//printf("CC,%d",rsp->ccode);
		//printf("LCD Data               : %x %x %x",
		//rsp->data[0], rsp->data[1], rsp->data[2]);

		//lcdstatus= (LCD_STATUS* ) rsp->data; commented for testing 
		lcdstatus->vKVM_status=rsp->data[1];
		lcdstatus->lock_status=rsp->data[2];

		return 0;
}

void ipmi_lcd_status_print( LCD_STATUS* lcdstatus)
{
		switch (lcdstatus->vKVM_status)
			{
			case 0x00: 
			printf("LCD KVM Status :Inactive\n");
			break;
			case 0x01: 
			printf("LCD KVM Status :Active\n");
			break;
			default:
			printf("LCD KVM Status :Invalid Status\n");
		    
			break;
			}			

		switch (lcdstatus->lock_status)
			{
			case 0x00: 
			printf("LCD lock Status :View and modify\n");
			break;
			case 0x01: 
			printf("LCD lock Status :View only\n");
			break;
			case 0x02:
			printf("LCD lock Status :disabled\n");
			break;
			default:
			printf("LCD lock Status :Invalid\n");
			break;
			}
		
}
/* ipmi_lcd_get_status  -  gets lcd access mode and vkvm active status 
 *  *
 *   * @intf:	ipmi interface
 *    * returns -1 on error
 *     * returns 0 if successful
 *      * returns 1 if device not present
 *       */


static int
ipmi_lcd_get_status(struct ipmi_intf * intf )
{
	int rc=0;
	
	LCD_STATUS * lcdstatus;
    LCD_STATUS  lcdstatus1;
    lcdstatus=&lcdstatus1;
 	rc =ipmi_lcd_get_status_val( intf, lcdstatus);
    if (rc <0)
		return -1;
	ipmi_lcd_status_print(lcdstatus);

	return rc;
	
}
/* ipmi_lcd_set_kvm  -  set vkvm active status 
 *  *
 *   * @intf:	ipmi interface
 *    * returns -1 on error
 *     * returns 0 if successful
 *      * returns 1 if device not present
 *       */
static int
ipmi_lcd_set_kvm(struct ipmi_intf * intf, char status)
{
	#define LSCC_DATA_LEN 2
	LCD_STATUS lcdstatus;
	int rc=0;
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[5];
	rc=ipmi_lcd_get_status_val(intf,&lcdstatus);
	if (rc < 0)
		return -1;
	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_SET_SYS_INFO;
	req.msg.data_len = 5;
	req.msg.data = data;
	data[0] = IPMI_DELL_LCD_STATUS_SELECTOR;
	data[1] = status;			/* active- incative*/
	data[2] = lcdstatus.lock_status;	/* full-veiw-locked */
	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		lprintf(LOG_ERR, " Error setting LCD status");
		rc= -1;
	}else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb)) {
                        lprintf(LOG_ERR, " Error getting LCD status: Command not supported on this system.");
                        return -1;
       	} else if (rsp->ccode > 0) {
		lprintf(LOG_ERR, " Error setting LCD status: %s",
			val2str(rsp->ccode, completion_code_vals));

		rc= -1;
	}

	return rc;
}
/* ipmi_lcd_set_lock  -  set lcd access mode
 *  *
 *   * @intf:	ipmi interface
 *    * returns -1 on error
 *     * returns 0 if successful
 *      * returns 1 if device not present
 *       */
static int
ipmi_lcd_set_lock(struct ipmi_intf * intf,  char lock)
{
	#define LSCC_DATA_LEN 2
	LCD_STATUS lcdstatus;
 	int rc =0;
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[5];
	rc=ipmi_lcd_get_status_val(intf,&lcdstatus);
	if (rc < 0)
		return -1;
	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_SET_SYS_INFO;
	req.msg.data_len = 5;
	req.msg.data = data;
	data[0] = IPMI_DELL_LCD_STATUS_SELECTOR;
	data[1] = lcdstatus.vKVM_status;			// active- incative
	data[2] = lock;			// full- veiw-locked 
	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL)
	{
		lprintf(LOG_ERR, " Error setting LCD status");
		rc= -1;
	}
	if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb)) 
	{
        lprintf(LOG_ERR, " Error getting LCD status: Command not supported on this system.");
        return -1;
    }
	else if (rsp->ccode > 0) 
	{
		lprintf(LOG_ERR, " Error setting LCD status: %s",
			val2str(rsp->ccode, completion_code_vals));

		rc= -1;
	}

	return rc;
}
/*Om6.0 LCD feature ajay*/
static int 
ipmi_lcd_set_single_line_text (struct ipmi_intf * intf, char * text)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[18];
	int bytes_to_store = strlen(text);
	int bytes_stored = 0;
	int ii;
	int rc = 0;
       if (bytes_to_store>IPMI_DELL_LCD_STRING_LENGTH_MAX)
        {
            lprintf(LOG_ERR, " Out of range Max limit is 62 characters");
           return 1;
           
        }
       else
        {
	bytes_to_store = MIN(bytes_to_store, IPMI_DELL_LCD_STRING_LENGTH_MAX);
	for (ii = 0; ii < 4; ii++) {
		/*first block, 2 bytes parms and 14 bytes data*/
		if (0 == ii) {
			int size_of_copy =  
				MIN((bytes_to_store - bytes_stored), IPMI_DELL_LCD_STRING1_SIZE);
			if (size_of_copy < 0)		/* allow 0 string length*/
				break;
			req.msg.netfn = IPMI_NETFN_APP;
			req.msg.cmd = IPMI_SET_SYS_INFO;
			req.msg.data_len = size_of_copy + 4; /* chars, selectors and sizes*/
			req.msg.data = data;
			data[0] = IPMI_DELL_LCD_STRING_SELECTOR;
			data[1] = ii;				/* block number to use (0)*/
			data[2] = 0;				/*string encoding*/
			data[3] = bytes_to_store;	/* total string length*/
			memcpy (data+4, text+bytes_stored, size_of_copy);
			bytes_stored += size_of_copy;
		} else {
			int size_of_copy =  
				MIN((bytes_to_store - bytes_stored), IPMI_DELL_LCD_STRINGN_SIZE);
			if (size_of_copy <= 0)
				break;
			req.msg.netfn = IPMI_NETFN_APP;
			req.msg.cmd = IPMI_SET_SYS_INFO;
			req.msg.data_len = size_of_copy + 2;
			req.msg.data = data;
			data[0] = IPMI_DELL_LCD_STRING_SELECTOR;
			data[1] = ii;				/* block number to use (1,2,3)*/
			memcpy (data+2, text+bytes_stored, size_of_copy);
			bytes_stored += size_of_copy;
		}

		rsp = intf->sendrecv(intf, &req);
		if (rsp == NULL) {
			lprintf(LOG_ERR, " Error setting text data");
			rc = -1;
		} else if (rsp->ccode > 0) {
			lprintf(LOG_ERR, " Error setting text data: %s",
				val2str(rsp->ccode, completion_code_vals));
			rc = -1;
		}
	}
        }
	return rc;
}

static int
ipmi_lcd_set_text(struct ipmi_intf * intf, char * text, int line_number)
{
	int rc = 0;

	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];
	IPMI_DELL_LCD_CAPS * lcd_caps;

	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_GET_SYS_INFO;
	req.msg.data_len = 4;
	req.msg.data = data;
	data[0] = 0;				/* get parameter*/
	data[1] = IPMI_DELL_LCD_GET_CAPS_SELECTOR;
	data[2] = 0;				/* set selector (n/a)*/
	data[3] = 0;				/* block selector (n/a)*/

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) 
	{
		lprintf(LOG_ERR, " Error getting LCD capabilities");
		return -1;
	}
	else if (rsp->ccode > 0) 
	{
		lprintf(LOG_ERR, " Error getting LCD capabilities: %s",
			val2str(rsp->ccode, completion_code_vals));

		return -1;
	}

	lcd_caps = (IPMI_DELL_LCD_CAPS *)(void *)rsp->data;

	if (lcd_caps->number_lines > 0) {
		rc = ipmi_lcd_set_single_line_text (intf, text);
	} else {
		lprintf(LOG_ERR, "LCD does not have any lines that can be set");
		rc = -1;
	}


	return rc;
}

int
ipmi_lcd_configure_wh (struct ipmi_intf * intf, uint32_t  mode ,uint16_t lcdquallifier, uint8_t errordisp, int8_t line_number, char * text)
{
	int rc = 0;


	if (IPMI_DELL_LCD_CONFIG_USER_DEFINED == mode)
		/* Any error was reported earlier. */
		rc = ipmi_lcd_set_text(intf, text, line_number);
   

	if (rc == 0)
              
		rc = ipmi_lcd_set_configure_command_wh (intf, mode ,lcdquallifier,errordisp);
 
	return rc;
}


int
ipmi_lcd_configure (struct ipmi_intf * intf, int command, int8_t line_number, char * text)
{
	int rc = 0;


	if (IPMI_DELL_LCD_CONFIG_USER_DEFINED == command)
		// Any error was reported earlier. 
		rc = ipmi_lcd_set_text(intf, text, line_number);

	if (rc == 0)
		rc = ipmi_lcd_set_configure_command (intf, command);

	return rc;
}


/* prints the 'id' portion of the sysinfo command
 */
static int 
ipmi_sysinfo_id(struct ipmi_intf * intf, int type)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	int rc = 0;
	struct ipm_devid_rsp * dev_id;
	struct sdr_repo_info_rs * repo_info;
	uint8_t ipmi_version;
																// ipmi get system info APP fn=06 cmd=01
	/* 20 80 01 52 51 9f a2 02 00 00 00 00 00 00 00 <- raw data*/
	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = BMC_GET_DEVICE_ID;
	req.msg.data = NULL;
	req.msg.data_len = 0;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		printf(" Error getting device ID (No Response)\n");
		return -1;
	}
	if (rsp->ccode > 0) {
		printf(" Error getting device ID (%s)\n",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}

	dev_id = (struct ipm_devid_rsp *)(void *)rsp->data;

	printf("Device ID           : %0.2d\n", dev_id->device_id);
	printf("Device Revision     : %0.2d\n", dev_id->device_revision & 0x7F);
	printf("Firmware Version    : %d.%0.2x\n", dev_id->fw_rev1 & 0x7F, 
		dev_id->fw_rev2); /* BCD value. byte->hex conversion used.*/
	printf("IPMI Version        : %d.%d\n", dev_id->ipmi_version & 0x0F, dev_id->ipmi_version >> 4);
	ipmi_version = dev_id->ipmi_version;


	printf("Manufacturer ID     : %lu\n", 
		(uint32_t)(dev_id->manufacturer_id[2] & 0x0f) * 65536 + 
		(uint32_t)(dev_id->manufacturer_id[1] * 256) + 
		(uint32_t)dev_id->manufacturer_id[0]);
	printf("Product ID          : %d\n", dev_id->product_id[1]*256 + dev_id->product_id[0]);

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = BMC_GET_SELF_TEST;
	req.msg.data = NULL;
	req.msg.data_len = 0;
	rsp = intf->sendrecv(intf, &req);

	if (rsp->data[0] == 0x55) {
	printf("Status              : No error\n",-1);
	}else if (rsp->data[0] == 0x56) { 
	printf("Status              : Self Test Function not implemented in this controller\n"); 
	}else if (rsp->data[0] == 0x58) 
	{ printf("Status              : Fatal Hardware Error (BMC is inoperative)\n"); 
	}else if (rsp->data[0] == 0x57) { 	
		printf("Status              : ");
		if (rsp->data[1] & 0x01) {printf("Controller operational firmware corrupte "); }
		if (rsp->data[1] & 0x02) {printf("Controller update 'boot block' firmware corrupte "); }
		if (rsp->data[1] & 0x04) {printf("Internal Use Area of BMC FRU corrupted "); }
		if (rsp->data[1] & 0x08) {printf("SDR Repository Empty "); }
		if (rsp->data[1] & 0x10) {printf("IPMB signal lines do not respond "); }
		if (rsp->data[1] & 0x20) {printf("Cannot access BMC FRU device "); }
		if (rsp->data[1] & 0x40) {printf("Cannot access SDR Repository "); }
		if (rsp->data[1] & 0x80) {printf("Cannot access SEL device"); }
		printf("\n");
	}
	else if (rsp->data[0] != 0xFF) { printf("Status              : Internal Failure. Refer to particular device's specification for definition\n"); }
	else { printf("Status              : \n"); }

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_SDR_REPO_INFO;
	req.msg.data = NULL;
	req.msg.data_len = 0;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		printf(" Error getting SDR version (No Response)\n");
		return -1;
	}
	if (rsp->ccode > 0) {
		printf(" Error getting SDR version (%s)\n",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}
	
	repo_info = (struct sdr_repo_info_rs *)(void *)rsp->data;
	printf("SDR Version         : %d.%d\n", repo_info->version & 0x0F, repo_info->version >> 4);


	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = BMC_GET_GUID;
	req.msg.data = NULL;
	req.msg.data_len = 0;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		printf(" Error getting device GUID (No Response)\n");
		return -1;
	}
	if (rsp->ccode > 0) {
		printf(" Error getting device GUID (%s)\n",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}
	printf("GUID                : %.2X%.2X%.2X%.2X-%.2X%.2X-%.2X%.2X-%.2X%.2X-%.2X%.2X%.2X%.2X%.2X%.2X\n",
		rsp->data[15], rsp->data[14], rsp->data[13], rsp->data[12],
		rsp->data[11], rsp->data[10], rsp->data[9], rsp->data[8], 
		rsp->data[7], rsp->data[6], rsp->data[5], rsp->data[4], 
		rsp->data[3], rsp->data[2], rsp->data[1], rsp->data[0]);


	rc = ipmi_sysinfo_fru(intf, 0, ipmi_version);
	return rc;
}

/* prints the 'fru' portion of the sysinfo command
 */
static int
ipmi_sysinfo_fru(struct ipmi_intf * intf, uint8_t id, uint8_t ipmi_version)
{
	struct ipmi_rs * rsp;
	struct ipmi_rq req;
	struct fru_info fru;
	struct fru_header header;
	uint8_t msg_data[4];

	

	memset(&fru, 0, sizeof(struct fru_info));
	memset(&header, 0, sizeof(struct fru_header));

	/*
	 * get info about this FRU
	 */
	memset(msg_data, 0, 4);
	msg_data[0] = id;

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_FRU_INFO;
	req.msg.data = msg_data;
	req.msg.data_len = 1;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		printf(" Device not present (No Response)\n");
		return -1;
	}
	if (rsp->ccode > 0) {
		printf(" Device not present (%s)\n",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}

	fru.size = (rsp->data[1] << 8) | rsp->data[0];
	fru.access = rsp->data[2] & 0x1;

	lprintf(LOG_DEBUG, "fru.size = %d bytes (accessed by %s)",
		fru.size, fru.access ? "words" : "bytes");

	if (fru.size < 1) {
		lprintf(LOG_ERR, " Invalid FRU size %d", fru.size);
		return -1;
	}

	/*
	 * retrieve the FRU header
	 */
	msg_data[0] = id;
	msg_data[1] = 0;
	msg_data[2] = 0;
	msg_data[3] = 8;

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_FRU_DATA;
	req.msg.data = msg_data;
	req.msg.data_len = 4;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		printf(" Device not present (No Response)\n");
		return 1;
	}
	if (rsp->ccode > 0) {
		printf(" Device not present (%s)\n",
		       val2str(rsp->ccode, completion_code_vals));
		return 1;
	}

	if (verbose > 1)
		printbuf(rsp->data, rsp->data_len, "FRU DATA");

	memcpy(&header, rsp->data + 1, 8);

	if (header.version != 1) 
	{
		lprintf(LOG_ERR, " Unknown FRU header version 0x%02x",
			header.version);
		return -1;
	}

	/* offsets need converted to bytes
	 * but that conversion is not done to the structure
	 * because we may end up with offset > 255
	 * which would overflow our 1-byte offset field */



	char assettag[IPMI_DELL_SYSINFO_ASSET_TAG_LENGTH];
	char servicetag[IPMI_DELL_SYSINFO_SERVICE_TAG_LENGTH] = {0};
	char hostname[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};
	char productModel[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};
	char osName[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};
	char biosversion[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};

	/* Print board area */
	if ((header.offset.board*8) >= sizeof(struct fru_header))
		ipmi_sysinfo_print_board(intf, &fru, id, header.offset.board*8, ipmi_version);

	/* Print the rest of the sysinfo command */
	if (ipmi_version == 0x51) {
		
/*		all the remaining info for ipmi 1.5 systems is printed in the ipmi_sysinfo_print_board
		function call above. */
	} else {
		/* for IPMI 2.0 we get the remaining info from the "get system info" IPMI command rather
		 * than from the 'get fru data' command */


		ipmi_lcd_get_platform_model_name(intf, hostname, IPMI_DELL_LCD_STRING_LENGTH_MAX, 
							IPMI_DELL_SYSINFO_HOST_NAME);
		printf("Host Name           : %s\n", hostname);


		ipmi_lcd_get_platform_model_name(intf, productModel, IPMI_DELL_LCD_STRING_LENGTH_MAX, 
						  IPMI_DELL_PLATFORM_MODEL_NAME_SELECTOR);
		printf("Product Model       : %s\n", productModel);

		ipmi_sysinfo_get_tag(intf, assettag, IPMI_DELL_SYSINFO_ASSET_TAG, 
										  IPMI_DELL_SYSINFO_ASSET_TAG_LENGTH);
		printf("Asset Tag           : %s\n", assettag);

		ipmi_sysinfo_get_tag(intf, servicetag, IPMI_DELL_SYSINFO_SERVICE_TAG, 
										  IPMI_DELL_SYSINFO_SERVICE_TAG_LENGTH);
		printf("Service Tag         : %s\n", servicetag);

		ipmi_lcd_get_platform_model_name(intf, biosversion, IPMI_DELL_LCD_STRING_LENGTH_MAX, 
											 IPMI_DELL_SYSINFO_BIOS_VERSION);
		printf("BIOS Version        : %s\n", biosversion);
		   
		ipmi_lcd_get_platform_model_name(intf, osName, IPMI_DELL_LCD_STRING_LENGTH_MAX, 
											 IPMI_DELL_SYSINFO_OS_NAME);
		printf("System OS Name      : %s\n", osName);

	}


	return 0;
}

/* fru_area_print_board  -  Print FRU Board Area
 *
 * @intf:	ipmi interface
 * @fru:	fru info
 * @id: 	fru id
 * @offset:	offset pointer
 */
static void
ipmi_sysinfo_print_board(struct ipmi_intf * intf, struct fru_info * fru,
		     uint8_t id, uint32_t offset, uint8_t ipmi_version)
{
	char * fru_area;
	char * bios_version; 
	char * prod_model;
	char * service_tag;
	char * asset_tag;
	uint8_t * fru_data;
	uint8_t language;
	uint32_t fru_len, area_len, i, j;
	uint16_t length;
	uint16_t bib_offset, bib_info_area, bib_length, bib_end;
        fru_area=NULL;
	j = 0;

	i = offset;
	fru_len = 0;


	fru_data = malloc(fru->size + 1);
	if (fru_data == NULL) {
		lprintf(LOG_ERR, " Out of memory!");
		return;
	}
	memset(fru_data, 0, fru->size + 1);

	/* read enough to check length field */
	if (read_fru_area(intf, fru, id, i, 2, fru_data) == 0)
		fru_len = 8 * fru_data[i + 1];
	if (fru_len <= 0) {
		free(fru_data);
		return;
	}

	fru_len = 0x200;
	i = 0;
	/* read in the full fru */
	if (read_fru_area(intf, fru, id, i, fru_len, fru_data) < 0) {
		free(fru_data);
		return;
	}


	language = fru_data[10];
	if (language == 0x00) {
		printf("Board Language Code : English\n");
	} else {
		printf("Board Language Code : Unknown\n");
	}


	i = 8;

	i++;	/* skip fru area version */
	area_len = fru_data[i++] * 8; /* fru area length */
	i++;	/* skip fru board language */
	i += 3;	/* skip mfg. date time */


	fru_area = get_fru_area_str(fru_data, &i);
	if (fru_area != NULL && strlen(fru_area) > 0) {
/*		printf(" Board Mfg             : %s\n", fru_area);*/
		free(fru_area);
	}

	fru_area = get_fru_area_str(fru_data, &i);
	if (fru_area != NULL && strlen(fru_area) > 0) {
		printf("Board Product Name  : %s\n", fru_area);
        
		free(fru_area);
	}

	fru_area = get_fru_area_str(fru_data, &i);
	if (fru_area != NULL && strlen(fru_area) > 0) {
		printf("Board Serial Number : %s\n", fru_area);

        	
		free(fru_area);
	}

	fru_area = get_fru_area_str(fru_data, &i);
	if (fru_area != NULL && strlen(fru_area) > 0) {

		printf("Board Part Number   : %s\n", fru_area);

		free(fru_area);
	}

	fru_area = get_fru_area_str(fru_data, &i);
	if (fru_area != NULL && strlen(fru_area) > 0) {
/*		if (verbose > 0)*/

if (*fru_area<48)*fru_area='\0';
            printf("Board FRU File ID   : %s\n", fru_area);
		free(fru_area);
	}

	/* read any extra fields */
	while ((fru_data[i] != 0xc1) && (i < offset + area_len))
	{
		int j = i;
		fru_area = get_fru_area_str(fru_data, &i);
		if (fru_area != NULL && strlen(fru_area) > 0) {
			printf(" Board Extra           : %s\n", fru_area);
			free(fru_area);
		}
		if (i == j)
			break;
	}

	if (ipmi_version == 0x51) {
 

		/* get host name */
		length = (fru_data[0x72] << 8) | (fru_data[0x71]);
		fru_area = malloc(length+1);
		memset(fru_area, 0, length+1);
		memcpy(fru_area, &fru_data[0x75], length);
		fru_area[length] = '\0';
		printf("Host Name           : %s\n", fru_area);
		free(fru_area);

		fru_area = ipmi_get_bib_info(fru_data, BIB_TYPE_PDRODUCT_MODEL);
		if (fru_area != NULL && strlen(fru_area) > 0) {
			printf("Product Model       : %s\n", fru_area);
			free(fru_area);
		}
		fru_area = ipmi_get_bib_info(fru_data, BIB_TYPE_ASSET_TAG);
		if (fru_area != NULL && strlen(fru_area) > 0) {
			printf("Asset Tag           : %s\n", fru_area);
			free(fru_area);
		}
		fru_area = ipmi_get_bib_info(fru_data, BIB_TYPE_SERVICE_TAG);
		if (fru_area != NULL && strlen(fru_area) > 0) {
			printf("Service Tag         : %s\n", fru_area);
			free(fru_area);
		}
		fru_area = ipmi_get_bib_info(fru_data, BIB_TYPE_BIOS_VERSION);
		if (fru_area != NULL && strlen(fru_area) > 0) {
			printf("BIOS version        : %s\n", fru_area);
			free(fru_area);
		}
    }


	free(fru_data);
}

/* get info from the BIB (BIOS information block) section of the FRU */
static char*  ipmi_get_bib_info(uint8_t * fru_data, uint8_t bib_type)
{
	uint16_t bib_offset, bib_info_area, bib_length, bib_end, length;
	uint32_t offset;
	char * bib_data;

		/* get BIB fields */
	if (fru_data[0x66] == 0xc3) {
		bib_offset = (fru_data[0x68] << 8) | fru_data[0x67];
	} else if (fru_data[0x69] == 0xc3) {
		bib_offset = (fru_data[0x6B] << 8) | fru_data[0x6A];
	}


	bib_info_area = bib_offset + 6;
	bib_length = (fru_data[bib_info_area+2]<<8) | fru_data[bib_info_area+1];
	bib_end = bib_info_area + bib_length;

	/* find BIB field */
	offset = bib_info_area;
	while ((fru_data[offset] != bib_type) && (offset <= bib_end)) {
		offset += fru_data[offset+1]+3;
	}
	if (fru_data[offset] == bib_type) {	/* bib info found */
		if (bib_type == BIB_TYPE_PDRODUCT_MODEL) {
			length = fru_data[offset+1]-6;
		} else {
			length = fru_data[offset+1];
		}
		
		bib_data = malloc(length+1);
		memset(bib_data, 0, length+1);
		if (bib_type == BIB_TYPE_PDRODUCT_MODEL) {
			memcpy(bib_data, &fru_data[offset+8], length);
		} else {
			memcpy(bib_data, &fru_data[offset+2], length);
		}
		bib_data[length] = '\0';
/*		printf("BIOS version: %s\n", bib_data);*/
	} else {
		bib_data = NULL;
	}
	return bib_data;
}

static int ipmi_sysinfo_get_tag(struct ipmi_intf * intf, char* tagstring, uint8_t tag_type, uint8_t length)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];
	IPMI_DELL_TAG * tagblock;
	int lcdstring_len = 0;
	int bytes_copied = 0;
/*	char tagstring[11] = {0};*/
	int ii;

		int bytes_to_copy;

		req.msg.netfn = IPMI_NETFN_APP;
		req.msg.cmd = IPMI_GET_SYS_INFO;
		req.msg.data_len = 4;
		req.msg.data = data;
		data[0] = 0;				/* get parameter*/
		data[1] = tag_type;/*to get asset tag*/
		data[2] = 0;
		data[3] = 0;

		rsp = intf->sendrecv(intf, &req);

		if (rsp == NULL) {
			lprintf(LOG_ERR, " Error getting asset tag");
			return -1;
		} else if (rsp->ccode > 0) {
			lprintf(LOG_ERR, " Error getting asset tag: %s",
				val2str(rsp->ccode, completion_code_vals));
			return -1;
		}

		tagblock = (IPMI_DELL_TAG *) (void *) rsp->data;
		memset(tagstring, 0,11);
		memcpy (tagstring, &tagblock->tag, length);

		return 0;

}



int ipmi_sel_print(struct ipmi_intf * intf, uint16_t count, int printOption)
{
	CSLFUSERAPI		pfnApiList;
	DECODING_METHOD	method;
	unsigned char	severity;
	uint8_t			index = 1;
	int				status = COMMON_STATUS_BAD_PARAMETER;
	unsigned short	TimeStrSize = LOG_DATE_STR_SIZE;
	unsigned short	DescriptionStrSize = EVENT_DESC_STR_SIZE;
	uint16_t		curr_id = 0;
	uint16_t		next_id;
	uint8_t			sel_entry[0xff];
	uint8_t			test;
	uint16_t		last = 0;
	uint16_t 		totalEntries = 0;
	uint16_t		start_id = 0;
	uint16_t		start = 1;
	int				rc;
	int				error_count;


	char LogTimeString[LOG_DATE_STR_SIZE];
	char DateString[LOG_DATE_STR_SIZE];
	char DescriptionString[EVENT_DESC_STR_SIZE];

	rc = ipmi_sdr_build_table(intf);
	if (rc == -1) {
		return rc;
	}

			struct ipmi_rs * rsp;
			struct ipmi_rq req;
			

			memset(&req, 0, sizeof(req));
			req.msg.netfn = IPMI_NETFN_STORAGE;
			req.msg.cmd = IPMI_CMD_GET_SEL_INFO;

			rsp = intf->sendrecv(intf, &req);
			if (rsp == NULL) {
				lprintf(LOG_ERR, " Error getting sel info");
				return -1;
			} else if (rsp->ccode > 0) {
				lprintf(LOG_ERR, " Error getting sel info: %s",
				val2str(rsp->ccode, completion_code_vals));
				return -1;
			}

			totalEntries = ((rsp->data[2] << 8) | rsp->data[1]);

	 	if (printOption == SEL_PRINT_FIRST) {
		
			if(count > totalEntries)
				last = totalEntries;
			else
				last = count;
		}
	    if (printOption == SEL_PRINT_LAST){
		
			if (count <=  totalEntries)
			{
				start_id = totalEntries - count;
				start = start_id+1;
		  	 	/*added here Meghna DF:170716*/
				index = start;
				last = totalEntries;
			}

/*defect ambigous beDF180792  FT32: ipmitool:Ambiguous behaviour with options of sel list. */
			else
			{
				start_id = 0;
				start = start_id+1;
		  	 	/*added here Meghna DF:170716*/
				index = start;
				last = totalEntries;
			}
		}
		if(printOption == SEL_PRINT_ALL){
		
				last = totalEntries;
		}


	/* set decoding method */
	rc = ipmi_sel_sysinfo(intf, &method);
	if (rc == -1) {
		return -1;
	}
	
	status = CSLFSetDecodingMethod(&method);
	if (COMMON_STATUS_SUCCESS == status) 
	{
		/* clear API structure */
		memset((void*) &pfnApiList, 0, sizeof(CSLFUSERAPI));
		pfnApiList.GetFirstSDR = sdr_get_first;
		pfnApiList.GetNextSDR = sdr_get_next;

		CSLFAttach(&pfnApiList);
		while (start <= last) 
		{

			error_count = 0;
			do {
				next_id = ipmi_sel_get_record(intf, start, sel_entry);
				error_count ++;
				if (next_id == NULL) 
				{
					printf("SDR is NULL and error_count is %d\n", error_count);
				}
			} while ( (next_id == NULL) && (error_count < 3) );
			
			if (next_id == NULL) {
				/*error message handled in ipmi_sel_get_record*/
				return -1;
			}
			start = next_id;
            {
			
				LogTimeString[0] = 0;
				DateString[0] = 0;
				DescriptionString[0] = 0;


				status = CSLFSELEntryToStr(
				(void*)sel_entry,
				0,
				LogTimeString,
				&TimeStrSize,
				DescriptionString,
				&DescriptionStrSize,
				&severity,
				(void*)intf);


			/*Convert date string*/
				status = CSLFSELUnixToCTime(LogTimeString, DateString);


				printf("Severity          : %s\n", g_StatusTable[severity]);
				printf("Date and Time     : %s\n", DateString);
				printf("Description       : %s\n\n", DescriptionString);
				
			}
		}
		CSLFDetach();
	}
	return 0;
}

static int make_int(const char *str, int *value)
{
	char *tmp=NULL;
	*value = strtol(str,&tmp,0);
	if ( tmp-str != strlen(str) )
	{
		return -1;
	}
	return 0;
}

extern int  ipmi_sel_get(struct ipmi_intf * intf, uint16_t id)
{
	uint16_t curr_id, next_id;
	SDRType * sdr;
	int rc;

	uint8_t *sdr_out;

	curr_id = 0x0000;
	next_id = 0x0000;
	while (curr_id != 0xFFFF) {
		curr_id = next_id;
	}

	uint16_t reserveId;

	DECODING_METHOD method;
	rc = ipmi_sel_sysinfo(intf, &method);
	if (rc == -1) {
		return -1;
	}
	printf("CompletionCode = %d\n", method.completionCode);
	printf("deviceId = %d\n", method.deviceID);
	printf("deviceRevision = %d\n", method.deviceRevision);
	printf("firmwareMajor = %d\n", method.firmwareMajor);
	printf("firmwareMinor = %d\n", method.firmwareMinor);
	printf("ipmiVersion = %d\n", method.ipmiVersion);
}


static char *
ipmi_sel_timestamp(uint32_t stamp)
{
	static char tbuf[40];
	time_t s = (time_t)stamp;
	memset(tbuf, 0, 40);
	strftime(tbuf, sizeof(tbuf), "%m/%d/%Y %H:%M:%S", gmtime(&s));				// edited here
	return tbuf;
}

uint16_t ipmi_sel_get_record(struct ipmi_intf * intf, uint16_t curr_id, uint8_t* sel_entry)
{
	struct ipmi_rq req;
	struct ipmi_rs * rsp;
	uint8_t msg_data[6];
	uint16_t next;
	int data_count;

	memset(msg_data, 0, 6);
	msg_data[0] = 0x00;	/* no reserve id, not partial get */
	msg_data[1] = 0x00;
	msg_data[2] = curr_id & 0xff;
	msg_data[3] = (curr_id >> 8) & 0xff;
	msg_data[4] = 0x00;	/* offset */
	msg_data[5] = 0xff;	/* length */

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = IPMI_CMD_GET_SEL_ENTRY;
	req.msg.data = msg_data;
	req.msg.data_len = 6;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
			lprintf(LOG_ERR, " Error getting sel record");
			return (-1) ;
	} else if (rsp->ccode > 0) {
			lprintf(LOG_ERR, " Error getting sel record: %s",
				val2str(rsp->ccode, completion_code_vals));
			return (-1);
	}

	/* save next entry id */
	next = (rsp->data[1] << 8) | rsp->data[0];

	memcpy((void*)sel_entry, (void*)&rsp->data[2], 16);

	return next;
}

int ipmi_sel_sysinfo(struct ipmi_intf *intf, DECODING_METHOD *method)
{
	struct ipmi_rs *rsp;
	struct ipmi_rq req;
	uint8_t data_in[16];
	struct ipm_devid_rsp * dev_id;

	memset(&req, 0, sizeof (req));

	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = BMC_GET_DEVICE_ID;
	req.msg.data = NULL;
	req.msg.data_len = 0;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
			lprintf(LOG_ERR, " Error getting system info");
			return -1;
		} else if (rsp->ccode > 0) {
			lprintf(LOG_ERR, " Error getting system info: %s",
				val2str(rsp->ccode, completion_code_vals));
			return -1;
		}

	method->completionCode = rsp->ccode;
	method->deviceID		= rsp->data[0];
	method->deviceRevision	= rsp->data[1];
	method->firmwareMajor	= rsp->data[2];
	method->firmwareMinor	= rsp->data[3];
	method->ipmiVersion		= rsp->data[4];
	method->supportFlags	= rsp->data[5];
	method->manufactureID[0]= rsp->data[6];
	method->manufactureID[1]= rsp->data[7];
	method->manufactureID[2]= rsp->data[8];
	method->productID[0]	= rsp->data[9];
	method->productID[1]	= rsp->data[10];
	method->auxiliary[0]	= rsp->data[11];
	method->auxiliary[1]	= rsp->data[12];
	method->auxiliary[2]	= rsp->data[13];
	method->auxiliary[3]	= rsp->data[14];

	return 1;


}

/* ipmi_sdr_get_reservation  -  Obtain SDR reservation ID
 *
 * @intf:	ipmi interface
 * @reserve_id:	pointer to short int for storing the id
 *
 * returns 0 on success
 * returns -1 on error
 */
int
ipmi_sdr_reserve(struct ipmi_intf *intf, uint16_t * reserve_id)
{
	struct ipmi_rs *rsp;
	struct ipmi_rq req;

	/* obtain reservation ID */
	memset(&req, 0, sizeof (req));

		req.msg.netfn = IPMI_NETFN_STORAGE;

	req.msg.cmd = GET_SDR_RESERVE_REPO;
	rsp = intf->sendrecv(intf, &req);
	/* be slient for errors, they are handled by calling function */
	if (rsp == NULL)
		return -1;
	if (rsp->ccode > 0)
		return -1;


	*reserve_id = ((struct sdr_reserve_repo_rs *) &(rsp->data))->reserve_id;
	lprintf(LOG_DEBUG, "SDR reservation ID %04x", *reserve_id);

	return 0;
}

/* Read all SDR data into memory */
static int ipmi_sdr_build_table(struct ipmi_intf *intf)
{
	SDRType* sdr_cur;
	SDRType* sdr_next;
	IPMISDR current;
	IPMISDR next;
	IPMISDR sdr_out;
	sdr_cur = &current;
	sdr_next = &next;
	int rc;
	int i = 0;
	int j;
	uint8_t* test;
	int count;

	count = 0;
	do {
		rc = ipmi_build_sdr_first(intf, SDR[0].data);
		count ++;
	} while ( (rc == -1) && (count < 3) );
	if (rc == -1) 
	{
		printf("Error: Cannot access SDR info:\n");
		return rc;
	}

	i = 0;

	do {

		count = 0;
		do {
			rc = ipmi_build_sdr_next(intf, SDR[i].data, SDR[i+1].data);
			count++;
		} while ( (rc == -1) && (count < 3) );
		 if (rc == -1) {
            		printf("Error: Cannot access SDR info\n");

			 return rc;
		 }
		 
		 i++;
	}while ((rc != 1) && (i<MAX_SDR_ENTRIES));

	if (i >= MAX_SDR_ENTRIES) {
		printf("Error: SDR number limit exceeded.\n");
		return -1;
	}

	totalSDRCount = i;

	return 0;

}

static int ipmi_build_sdr_first(struct ipmi_intf *intf, uint8_t *data)
{
	uint16_t reserve_id;
	int rc;
	SDRType* sdr_out;
	uint8_t* test;

	rc = ipmi_sdr_reserve(intf, &reserve_id);
	if (rc == -1) {
		return rc;
	}

	rc = ipmi_sdr_get(intf, reserve_id, data, 0x00);


	
	return rc;

}

int ipmi_build_sdr_next(struct ipmi_intf *intf, uint8_t *sdr_in, uint8_t *data)
{
	uint16_t reserve_id, record_id;
	int rc;
	SDRType *sdr_out;
	struct ipmi_rs *rsp;
	struct ipmi_rq req;
	uint8_t msg_data[6];




	memset(&req, 0, sizeof (req));



	/* Get next record ID */
	memset(msg_data, 0, 6);
	msg_data[0] = 0x00;
	msg_data[1] = 0x00;
	msg_data[2] = sdr_in[0]; /* record id */
	msg_data[3] = sdr_in[1]; /* record id */
	msg_data[4] = 0x00;	/* offset */
	msg_data[5] = 0x0f;	/* length */

	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_SDR;
	req.msg.data = msg_data;
	req.msg.data_len = 6;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) 
	{
		return -1;
	}
	else if (rsp->ccode > 0) 
	{
		return -1;
	}

	record_id = (rsp->data[1] << 8) | rsp->data[0];

	if (record_id != 0xFFFF) 
	{

		rc = ipmi_sdr_reserve(intf, &reserve_id);
		if (rc == -1) 
		{
			return rc;
		}

		rc = ipmi_sdr_get(intf, reserve_id, data, record_id);
		if (rc == -1) 
		{
			return rc;
		}

		return 0;
	} 
	else 
	{
		return 1;
	}


}


int ipmi_sdr_get(struct ipmi_intf *intf, uint16_t reserveId, uint8_t* datapt, uint16_t recordId)
{
	#define MAX_BUF_SIZE 16

	struct ipmi_rs *rsp;
	struct ipmi_rq req;
	uint8_t msg_data[6];
	uint16_t curr_id = 0;
	uint8_t orgremain;
	uint8_t length;
	uint8_t offset;
	uint8_t remain;
	int i;
	int rc;

	memset(&req, 0, sizeof (req));


	/* Get header info */
	memset(msg_data, 0, 6);
	msg_data[0] = reserveId & 0xff;	
	msg_data[1] = (reserveId >> 8) & 0xff;
	msg_data[2] = recordId & 0xff;
	msg_data[3] = (recordId >> 8) & 0xff;
	msg_data[4] = 0x00;	/* offset */
	msg_data[5] = 0x05;	/* length */

	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_SDR;
	req.msg.data = msg_data;
	req.msg.data_len = 6;

	

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
			return -1;
		} else if (rsp->ccode > 0) {
			return -1;
		}

	memcpy((uint8_t*)datapt, (uint8_t*)rsp->data + 2, 5);



	orgremain = datapt[4];
	remain = datapt[4];
	offset = 5;
	while (remain > 0) {
		if (remain  < MAX_BUF_SIZE) {
			length = remain;
		} else {
			length = MAX_BUF_SIZE;
		}
		memset(msg_data, 0, 6);
		msg_data[0] = reserveId & 0xff;	
		msg_data[1] = (reserveId >> 8) & 0xff;
		msg_data[2] = recordId & 0xff;
		msg_data[3] = (recordId >> 8) & 0xff;
		msg_data[4] = offset;	/* offset */
		msg_data[5] = length;	/* length */

		req.msg.netfn = IPMI_NETFN_STORAGE;
		req.msg.cmd = GET_SDR;
		req.msg.data = msg_data;
		req.msg.data_len = 6;

		rsp = intf->sendrecv(intf, &req);
		if (rsp == NULL) {
			return -1;
		} else if (rsp->ccode > 0) {
			return -1;
		}

		memcpy(datapt+offset, rsp->data + 2, length);

		offset += length;
		remain -= length;


	}




 rc = 0;
	return rc;

}

SDRType* sdr_get_first()
{
	currentSDR = 0;
	return ((SDRType*)(SDR[0].data));
}

SDRType* sdr_get_next(SDRType* pPreviousSDR)
{
	if ((currentSDR + 1) < totalSDRCount) {
		currentSDR++;
		return ((SDRType*)(SDR[currentSDR].data));
	} else {
		return NULL;
	}
}





static uint16_t
ipmi_sel_reserve(struct ipmi_intf * intf)
{
	struct ipmi_rs * rsp;
	struct ipmi_rq req;

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = IPMI_CMD_RESERVE_SEL;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		lprintf(LOG_WARN, "Unable to reserve SEL");
		return 0;
	}
	if (rsp->ccode > 0) {
		printf("Unable to reserve SEL: %s",
		       val2str(rsp->ccode, completion_code_vals));
		return 0;
	}

	return (rsp->data[0] | (rsp->data[1] << 8));
}

static int ipmi_sensor_print(SDRType* pSdr, int* pSensorIndex, struct ipmi_intf * intf)
{
    CSDDUSERAPI      pfnApiList;
    int              count;
    short            typeSize;
    short            nameSize;
    short            unitSize;
    unsigned char    readingType;
    unsigned char    type;
    char             szUnits[16];
    long             sensorReading;
    unsigned short   sensorState;
    short            readSize;
    short            stateSize;
    unsigned char    thresholdMask;
    SensorThrInfo    thrData;
    SensorThrStrType thrStrData;
    char             szState[SENSOR_STATE_STR_SIZE];
    short            severity;
    char             szName[SENSOR_NAME_STR_SIZE];
    char             szType[SENSOR_TYPE_STR_SIZE];
    char             szReading[SENSOR_READING_STR_SIZE];
    CSDDSensorList   list;
    int              status  = COMMON_STATUS_BAD_PARAMETER;
	IPMISDR*		pSdrPrint;


    /* Clear API structure */
    memset(&pfnApiList, 0, sizeof(CSDDUSERAPI));

    pfnApiList.GetFirstSDR         = sdr_get_first;
    pfnApiList.GetNextSDR          = sdr_get_next;
    pfnApiList.GetSensorReading    = ipmi_sensor_get_sensor_reading;

    CSDDAttach(&pfnApiList);


    if (NULL != pSdr)
    {
        memset((void*)&list, 0, sizeof(CSDDSensorList));
        status = CSDDGetSensorsTobeMonitored(&list, pSdr, NULL, (void*)intf);
        /* A failure here means no sensor present/disable for this SDR */
        if (COMMON_STATUS_SUCCESS == status)
        {
            for(count = 0; count < list.sensorCount; count++)
            { 
                typeSize        = SENSOR_TYPE_STR_SIZE;
                nameSize        = SENSOR_NAME_STR_SIZE;
                unitSize        = 16;
                stateSize       = SENSOR_STATE_STR_SIZE;
                readSize        = SENSOR_READING_STR_SIZE;
                /* initialize string buffers */
                szState[0]      = 0;
                szType[0]       = 0;
                szReading[0]    = 0;
                szUnits[0]      = 0;

                /* GetSensor Static Information */
                status = CSDDGetSensorStaticInformation(pSdr,0,&readingType,&type,
                             &nameSize, szName, &typeSize, szType, &unitSize,
                             szUnits, list.sensorNumber[count], (void*)intf);
				

                /* Did we encounter an error? */
                if (COMMON_STATUS_SUCCESS != status)
                {
				//	printf("Error1\n");
                    break;
                }
				memset(&sensorReading, 0, sizeof(sensorReading));
                status = CSDDGetSensorDynamicInformation(pSdr, &sensorReading,
                            &sensorState, &readSize, szReading, &stateSize,
                            szState, &severity,  list.sensorNumber[count], (void*)intf);

                /* Did we encounter an error? */
                if (COMMON_STATUS_SUCCESS != status)
                {
					printf("Error getting dynamic information: %x\n", status);
                    break;
                }

                /* Output common decoded data */
				
				(*pSensorIndex) ++;
				pSdrPrint = (IPMISDR*)pSdr;
				if (pSdrPrint->type.type1.entityID != 0x22) {
					
					if (IPMI_READING_TYPE_THRESHOLD == readingType)
					{
						printf("Index            : %d\n", *pSensorIndex);
						printf("Sensor ID Number : %d\n", pSdrPrint->type.type1.sensorNum);
						printf("Status           : %s\n", g_StatusTable[severity]);
						printf("Probe Name       : %s\n", szName);
						printf("Reading          : %s %s\n", szReading, szUnits);
#if 0
                                        memset((void*)&thrData, 0, sizeof(SensorThrInfo));
						memset((void*)&thrStrData, 0, sizeof(SensorThrStrType));
						thresholdMask = 0;        
					
						status = CSDDGetSensorThresholds(pSdr, &thresholdMask, &thrData,
                                &thrStrData, 0, (void*)intf);

						if (COMMON_STATUS_SUCCESS != status)
						{
							printf("Error3\n");
							break;
						}

						printf ("Minimum Warning Threshold      : %s %s\n", thrStrData.lncThr,szUnits);
						printf ("Maximum Warning Threshold      : %s %s\n", thrStrData.uncThr,szUnits);
						printf ("Minimum Failure Threshold      : %s %s\n", thrStrData.lcThr,szUnits);
						printf ("Maximum Failure Threshold      : %s %s\n", thrStrData.ucThr,szUnits);
#endif 

						}
						else
						{
							printf("Index            : %d\n", *pSensorIndex);
							printf("Sensor ID Number : %d\n", pSdrPrint->type.type1.sensorNum);
							printf("Status           : %s\n", szState); 
							printf("Probe Name       : %s\n", szName);
					}
					printf("\n");
				}
			}
        }
    }
    else
        {
            printf("sensor not found");
        }
    CSDDDetach();
    return status;
}



static int ipmi_sensor(struct ipmi_intf* intf, uint8_t sensorNumIn)
{
	int rc = 0;
	int sensorIndex = 0;
	int* pSensorIndex;
	pSensorIndex = &sensorIndex;
	IPMISDR*        pRecord;
	int i;

	rc = ipmi_sdr_build_table(intf);
	if (rc == -1) {
		return rc;
	}

	i = 0;
	while (i < totalSDRCount) {
		pRecord = (IPMISDR*)SDR[i].data;
		if (sensorNumIn == 0xff && rc != -1) {
			rc = ipmi_sensor_print((SDRType*)SDR[i].data, &sensorIndex, intf);
		}else if (pRecord->type.type1.sensorNum == sensorNumIn){
			rc = ipmi_sensor_print((SDRType*)SDR[i].data, &sensorIndex, intf);
		}
		i++;
	}




	return rc;
}


static int ipmi_sensor_get(struct ipmi_intf* intf, int argc, char **argv)
{
	int rc = 0;
	int sensorIndex = 0;
	int* pSensorIndex;
	pSensorIndex = &sensorIndex;
	IPMISDR*        pRecord;
	int i;
	int current_arg;
	int currIdInt;
	uint8_t currId;
       uint8_t currId_met=0;

	rc = ipmi_sdr_build_table(intf);
	if (rc == -1) {
		printf("failed in building SDR");
		
		return rc;
	}

	current_arg = 0;
	while (current_arg < argc) {
		make_int(argv[current_arg],&currIdInt);
		
		
		current_arg++;
	}

    if ((currIdInt > 255)||(currIdInt ==0))
    {
    	printf("Invalid ID : ID value is out of range\n");
        return -1;
    }
    currId = currIdInt;
	
    i = 0;
	while (i < totalSDRCount) 
	{
		pRecord = (IPMISDR*)SDR[i].data;
		if (pRecord->type.type1.sensorNum == currId)
		{
			rc = ipmi_sensor_print((SDRType*)SDR[i].data, &sensorIndex, intf);
                currId_met=1;
		}
		i++;
	} 
    //printf("currId_met%d\n",currId_met);
    if (currId_met==0)
    {
    	printf("Invalid Id: Id not found \n");
        currId_met=0;
    }
	return rc;
}

/* ipmi_sdr_get_sensor_reading  -  retrieve a raw sensor reading
 *
 * @intf:	ipmi interface
 * @sensor:	sensor id
 *
 * returns ipmi response structure
 */
int
ipmi_sensor_get_sensor_reading(unsigned char sensorOwner, unsigned char sensorNumber,
							   IPMISensorReadingType* sensorReadingData,
							   void* x)
{
	struct ipmi_rq req;
	struct ipmi_rs * rsp;
	struct ipmi_intf * intf;
	int rc = 0;
	uint8_t save_addr;



	intf = ((struct ipmi_intf*) x);

	memset(&req, 0, sizeof (req));
	req.msg.netfn = IPMI_NETFN_SE;
	req.msg.cmd = GET_SENSOR_READING;
	req.msg.data = &sensorNumber;
	req.msg.data_len = 1;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
			return 1;
		} else if (rsp->ccode > 0) {
			return 1;
		}

	memcpy(sensorReadingData, rsp->data, sizeof(IPMISensorReadingType));

	/* if there is an error transmitting ipmi command, return error*/
	if (rsp->ccode != 0) {
		rc = 1;
	}

	/* if sensor messages are disabled, return error*/
	if ((!(rsp->data[1]& 0xC0)) || ((rsp->data[1] & 0x20))) {
		rc =1;
	}



	return rc;
}

static int ipmi_powermgmt(struct ipmi_intf* intf)
{
	time_t now;
    struct tm* tm;
    char* dte;

	struct ipmi_rs * rsp;
	struct ipmi_rq req;
	uint8_t msg_data[2];
	uint32_t cumStartTimeConv;
	uint32_t cumReadingConv;
	uint32_t maxPeakStartTimeConv;
	uint32_t ampPeakTimeConv;
	uint16_t ampReadingConv;
	uint32_t wattPeakTimeConv;
	uint32_t wattReadingConv;
    uint32_t bmctimeconv;
    uint32_t * bmctimeconvval;

	IPMI_POWER_MONITOR* pwrMonitorInfo;

	
	char cumStartTime[26];
	char maxPeakStartTime[26];
	char ampPeakTime[26];
	char wattPeakTime[26];
    char bmctime[26];

	float cumReading;
	int ampReading;
	int wattReading;
	int ampReadingRemainder;

	now = time(0);
	tm = gmtime(&now);
	dte = asctime(tm);

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = IPMI_CMD_GET_SEL_TIME;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
			lprintf(LOG_ERR, " Error getting BMC time info.\n");
			return -1;
	} 
	if (rsp->ccode != 0) {
		printf("Error getting power management information, return code %x\n", rsp->ccode);
		return -1;
	}
      bmctimeconvval=(uint32_t*)rsp->data;
      bmctimeconv=*bmctimeconvval;

	/* get powermanagement info*/
	req.msg.netfn = 0x30;
	req.msg.lun = 0x0;
	req.msg.cmd = 0x9c;
	req.msg.data = msg_data;
	req.msg.data_len = 2;


		
	memset(msg_data, 0, 2);
/*	printf("##########netfn = %x\n", req.msg.netfn);*/
	msg_data[0] = 0x07;	
	msg_data[1] = 0x01;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
				lprintf(LOG_ERR, " Error getting power management information.\n");
				return -1;
	} 
	if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb)) {
		lprintf(LOG_ERR, " Error getting power management information: Command not supported on this system.");
		return -1;
	}else if (rsp->ccode != 0) {
		printf("Error getting power management information, return code %x\n", rsp->ccode);
		return -1;
	}



	pwrMonitorInfo = (IPMI_POWER_MONITOR*)rsp->data;
     
	cumStartTimeConv = pwrMonitorInfo->cumStartTime;
	cumReadingConv = pwrMonitorInfo->cumReading;
	maxPeakStartTimeConv = pwrMonitorInfo->maxPeakStartTime;
	ampPeakTimeConv = pwrMonitorInfo->ampPeakTime;
	ampReadingConv = pwrMonitorInfo->ampReading;
	wattPeakTimeConv = pwrMonitorInfo->wattPeakTime;
	wattReadingConv = pwrMonitorInfo->wattReading;

	ipmi_time_to_str(cumStartTimeConv, cumStartTime);
     
	ipmi_time_to_str(maxPeakStartTimeConv, maxPeakStartTime);
    ipmi_time_to_str(ampPeakTimeConv, ampPeakTime);
	ipmi_time_to_str(wattPeakTimeConv, wattPeakTime);
    ipmi_time_to_str(bmctimeconv, bmctime);
  
       

	now = time(0);

	int round;
	int round2;
	int remainder;


	remainder = (cumReadingConv % 1000);
	cumReadingConv = cumReadingConv / 1000;
	remainder = (remainder + 50) / 100;

	ampReading = ampReadingConv;
	ampReadingRemainder = ampReading%10;
	ampReading = ampReading/10;

	wattReading = wattReadingConv;

	printf("Power Tracking Statistics\n");
	printf("Statistic                  : Energy Consumption\n");
	printf("Measurement Start Time     : %s", cumStartTime);
	printf("Measurement Finish Time    : %s", bmctime);
    
    printf("Reading                    : %d.%d kWh\n\n", cumReadingConv, remainder);

	printf("Statistic                  : System Peak Power\n");
	printf("Measurement Start Time     : %s", maxPeakStartTime);
 	printf("Peak Time                  : %s", wattPeakTime);
	printf("Peak Reading               : %d W\n\n", wattReading);

   	printf("Statistic                  : System Peak Amperage\n");
	printf("Measurement Start Time     : %s", maxPeakStartTime);
 	printf("Peak Time                  : %s", ampPeakTime);
	printf("Peak  Reading              : %d.%d A\n", ampReading, ampReadingRemainder);


	return 0;

}

static int
ipmi_powermgmt_clear(struct ipmi_intf* intf,uint8_t clearValue)
{
	struct ipmi_rs * rsp;
	struct ipmi_rq req;
	uint8_t	clearType;
	uint8_t msg_data[3];

	if (clearValue) {
		clearType = 2;
	} else {
		clearType = 1;
	}

	/* clear powermanagement info*/
	req.msg.netfn = 0x30;
	req.msg.lun = 0;
	req.msg.cmd = 0x9d;
	req.msg.data = msg_data;
	req.msg.data_len = 3;

		
	memset(msg_data, 0, 3);
	msg_data[0] = 0x07;
	msg_data[1] = 0x01;
	msg_data[2] = clearType;



	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
				lprintf(LOG_ERR, " Error clearing power values.\n");
				return -1;
	} else if (rsp->ccode == 0xc1) {
		lprintf(LOG_ERR, " Error clearing power values, command not supported on this system.\n");
		return -1;
	} else if (rsp->ccode != 0){
		lprintf(LOG_ERR, " Error clearing power values: %s",
				val2str(rsp->ccode, completion_code_vals));
		return -1;
	}

}
//static int ipmi_get_power_supply_info(struct ipmi_intf* intf,int entity_id , int entity_instance , //IPMISDR*  pRecord,
int ipmi_get_power_headroom_command (struct ipmi_intf * intf,uint8_t unit)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
    uint64_t peakpowerheadroombtuphr;
    uint64_t instantpowerhearoom;

    req.msg.netfn = 0x30;
	req.msg.cmd = 0xBB;
	req.msg.data_len = 0;
	
	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		lprintf(LOG_ERR, " Error getting power headroom status");
		return -1;
	} else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb)){
		lprintf(LOG_ERR, " Error getting power headroom status: Command not supported on this system ");
			//val2str(rsp->ccode, completion_code_vals));
		return -1;
	} else if (rsp->ccode > 0) {
		lprintf(LOG_ERR, " Error getting power headroom status: %s",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}
	if(verbose>1)
	    printf("power headroom  Data               : %x %x %x %x ",
	 		/*need to look into */									rsp->data[0], rsp->data[1], rsp->data[2], rsp->data[3]);
	powerheadroom= *(( POWER_HEADROOM *)rsp->data);
	
	if(unit == btuphr)
	{
		peakpowerheadroombtuphr=watt_to_btuphr_conversion(powerheadroom.peakheadroom);
		instantpowerhearoom= watt_to_btuphr_conversion(powerheadroom.instheadroom);

        printf ("Name       : System Instantaneous Headroom\n");
		printf ("Reading    : %d BTU/hr\n",instantpowerhearoom);
        printf ("Name       : System Peak Headroom\n");
		printf ("Reading    : %d BTU/hr\n",peakpowerheadroombtuphr);
	}
	else
	{
        printf ("Name       : System Instantaneous Headroom\n");
		printf ("Reading    : %d W\n",powerheadroom.instheadroom);
        printf ("Name       : System Peak Headroom\n");
		printf ("Reading    : %d W\n",powerheadroom.peakheadroom);
	}

    return 0;
}
					
static int ipmi_get_power_supply_info(struct ipmi_intf* intf,IPMISDR*  pRecord,
					IPMI_POWER_SUPPLY_INFO* powersupplyinfo)

{
	struct ipmi_rs * rsp;
	struct ipmi_rq req={0};
	uint8_t msg_data[2];


	/* power supply rating command*/
	req.msg.netfn = 0x30;
	req.msg.lun = 0;
	req.msg.cmd = 0xb0;
	req.msg.data = msg_data;
	req.msg.data_len = 2;

		
	memset(msg_data, 0, 2);

	msg_data[0] = pRecord->type.type2.entityID;
	msg_data[1] = pRecord->type.type2.entityInstance;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL)
	{
		lprintf(LOG_ERR, " Error getting power supply info .\n");
		return -1;
	}
	else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb)) 
	{
		lprintf(LOG_ERR, "  Error getting power supply info: Command not supported on this system");
		return -1;
	}
	else if (rsp->ccode != 0)
	{
		lprintf(LOG_ERR, "  Error getting power supply info: %s",
				val2str(rsp->ccode, completion_code_vals));
		return -1;
	}
	if (verbose > 1)
	{
		printf("power supply info  data               : %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x%x%x\n",
							rsp->data[0], rsp->data[1], rsp->data[2], rsp->data[3], 
							rsp->data[4], rsp->data[5], rsp->data[6], rsp->data[7], 
							rsp->data[8], rsp->data[9], rsp->data[10], rsp->data[11], 
							rsp->data[12], rsp->data[13], rsp->data[14], rsp->data[15], 
							rsp->data[16], rsp->data[17], rsp->data[18], rsp->data[19]);
	}

	*powersupplyinfo =*((IPMI_POWER_SUPPLY_INFO*)(rsp->data));
	
    //status1= getpowersupplyoutputwattage(intf,&outputwatt1,&outputwatt2);
          
	return 0;
}


static int ipmi_print_power_supply_info(IPMISDR*  pRecord,int* pSensorIndex, struct ipmi_intf * intf,
		IPMI_POWER_SUPPLY_INFO* powersupplyinfo)
{


  CSDDUSERAPI      pfnApiList;
    int              count;
 
    short            typeSize;
    short            nameSize;
    short            unitSize;
    unsigned char    readingType;
    unsigned char    type;
    char             szUnits[16];
    long             sensorReading;
    unsigned short   sensorState;
    short            readSize;
    short            stateSize;
    unsigned char    thresholdMask;
    SensorThrInfo    thrData;
    SensorThrStrType thrStrData;
    char             szState[SENSOR_STATE_STR_SIZE];
    short            severity;
    char             szName[SENSOR_NAME_STR_SIZE];
    char             szType[SENSOR_TYPE_STR_SIZE];
    char             szReading[SENSOR_READING_STR_SIZE];
    CSDDSensorList   list;
    int              status  = COMMON_STATUS_BAD_PARAMETER;
	IPMISDR*		pSdrPrint;
	char sFrimwareVersion[MAX_POWER_FW_VERSION+1];


    /* Clear API structure */
    memset((void*)&pfnApiList, 0, sizeof(CSDDUSERAPI));

    pfnApiList.GetFirstSDR         = sdr_get_first;
    pfnApiList.GetNextSDR          = sdr_get_next;
    pfnApiList.GetSensorReading    = ipmi_sensor_get_sensor_reading;

    CSDDAttach(&pfnApiList);
	

    if (NULL != pRecord)
    {
        memset((void*)&list, 0, sizeof(CSDDSensorList));
        status = CSDDGetSensorsTobeMonitored(&list, pRecord, NULL, (void*)intf);
        /* A failure here means no sensor present/disable for this SDR */
        if (COMMON_STATUS_SUCCESS == status)
        {
            for(count = 0; count < list.sensorCount; count++)
            { 
                typeSize        = SENSOR_TYPE_STR_SIZE;
                nameSize        = SENSOR_NAME_STR_SIZE;
                unitSize        = 16;
                stateSize       = SENSOR_STATE_STR_SIZE;
                readSize        = SENSOR_READING_STR_SIZE;
                /* initialize string buffers */
                szState[0]      = 0;
                szType[0]       = 0;
                szReading[0]    = 0;
                szUnits[0]      = 0;

                /* GetSensor Static Information */
                status = CSDDGetSensorStaticInformation(pRecord,0,&readingType,&type,
                             &nameSize, szName, &typeSize, szType, &unitSize,
                             szUnits, list.sensorNumber[count], (void*)intf);
				

                /* Did we encounter an error? */
                if (COMMON_STATUS_SUCCESS != status)
                {
			     // printf("Get sensor data command  Unsucessful");		
                    break;
                }
				memset(&sensorReading, 0, sizeof(sensorReading));
                status = CSDDGetSensorDynamicInformation(pRecord, &sensorReading,
                            &sensorState, &readSize, szReading, &stateSize,
                            szState, &severity,  list.sensorNumber[count], (void*)intf);

                /* Did we encounter an error? */
                if (COMMON_STATUS_SUCCESS != status)
                {
					printf("Error getting dynamic information: %x\n", status);
                    break;
                }

                /* Output common decoded data */
				
				
				pSdrPrint = (IPMISDR*)pRecord;


				if (pSdrPrint->type.type2.sensorNum== 0x74)
				{
					
				
                	//rc = ipmi_sensor_print((SDRType*)SDR[i].data, &sensorIndex, intf);

					printf("----------------------------------------------------------\n");
					printf("Main System Chassis Power Supplies :%s\n",szState);
					printf("----------------------------------------------------------\n\n\n");
					printf("Power supply Redundacny : \n");
					//printf("Status           : %s\n", sensorState); 
					printf("Status    : %s\n", szState); 
					printf("Attribute : Redundancy Status\n");
					printf("Value 	  : %s\n",g_StatusTable[severity]);	
					printf("Attribute : Number of devices required for full redundancy \n");
					printf("Value 	  : 2\n\n");	
					printf("Individual Power Supply Elements\n\n");
				}
				else if ((pSdrPrint->type.type1.entityID != 0x22)&&(pSdrPrint->type.type2.sensorNum!= 0x74)) 
				{
					(*pSensorIndex) ++;
					printf("Index                   : %d\n", *pSensorIndex);
					//printf("Sensor ID Number : %d\n", pSdrPrint->type.type1.sensorNum);
					printf("Status                  : %s\n", szState); 
					printf("location                : %s \n",szName);
                    if (powersupplyinfo->Powersupplytype&0x01)
                    {
					printf("Type                    : DC\n");
                    }
                    else
                    {
        	        printf("Type                    : AC\n");
                    }
                    printf("Maximum Input Wattage   : %d W\n",powersupplyinfo->ratedWatts);
					printf("Maximum Output Wattage  : %d W\n",powersupplyinfo->ratedDCWatts);
					printf("Online Status           : %s\n", szState);
					printf("Power Monitoring capable: %s\n", g_StatusTable[severity]);
					strncpy(sFrimwareVersion,powersupplyinfo->FrimwareVersion,MAX_POWER_FW_VERSION);
					sFrimwareVersion[MAX_POWER_FW_VERSION] = 0;
					printf("Firmware Version        : %s\n",sFrimwareVersion);			
					//printf("power supply rated Amps               : %d A\n",powersupplyinfo->ratedAmps);

					
					printf("\n");
				}
			}
		}
    }
    CSDDDetach();
	
	
return status;

}

static int ipmi_get_power_supply_rating(struct ipmi_intf* intf)
{
	int rc = 0;
	int sensorIndex = 0;
	int* pSensorIndex;
	pSensorIndex = &sensorIndex;
	IPMISDR*        pRecord;
	IPMI_POWER_SUPPLY_INFO powersupplyinfo;
    char status=0;
	struct sdr_record_list *sdr;
	struct sdr_record_list *entry;;
	
	struct entity_id entity;
	int i=0;
	int index =0;

	rc = ipmi_sdr_build_table(intf);
	if (rc == -1) 
	{
		return rc;
	}

	i = 0;
	
	while (i < totalSDRCount)
	{
		pRecord = (IPMISDR*)SDR[i].data;
		if ((pRecord->type.type2.entityID == 0x07)&&(pRecord->type.type2.sensorNum==0x74))
		{
			rc=ipmi_print_power_supply_info(pRecord,&sensorIndex, intf,& powersupplyinfo);
            status=1;
			break;
		}
		i++;
	}
	i=0;

	while (i < totalSDRCount) 
	{
		pRecord = (IPMISDR*)SDR[i].data;
			
		if((pRecord->type.type2.entityID==0x0A)&&(pRecord->type.type2.sensorType==0x08))
		{
			if (rc != -1) 
			{
				index++;
				rc = ipmi_get_power_supply_info(intf,  pRecord,& powersupplyinfo);
    	        if(rc==0) 
					rc=ipmi_print_power_supply_info(pRecord,&sensorIndex, intf,& powersupplyinfo);
			}
    	    status=1;
        }
		i++;
	}

	if (status==0)
		printf("Could not find  Power supply sensor in SDR");
	
	return rc;
}
/*power consumption data*/
static int ipmi_get_power_consumption_data(struct ipmi_intf* intf,uint8_t unit)
{
    struct ipmi_rs * rsp;
	struct sdr_record_list *sdr;
	int readingbtuphr;
	int warning_threshbtuphr;
	int failuer_thresbtuphr;
	int status=0;
 
	sdr = ipmi_sdr_find_sdr_byid(intf, "System Level");

	
	rsp = ipmi_sdr_get_sensor_thresholds(intf,sdr->record.full->keys.sensor_num);

	if (rsp != NULL && rsp->ccode == 0)
    {   
		warning_threshbtuphr=sdr_convert_sensor_reading
					       (sdr->record.full, rsp->data[4]);
		failuer_thresbtuphr=sdr_convert_sensor_reading
					       (sdr->record.full, rsp->data[5]);						

		if (unit==btuphr)
		{
			//readingbtuphr= watt_to_btuphr_conversion(szReading);
			warning_threshbtuphr= watt_to_btuphr_conversion(warning_threshbtuphr);
			failuer_thresbtuphr= watt_to_btuphr_conversion( failuer_thresbtuphr);

		  //printf ("Reading 			: %d BTU/hr\n",readingbtuphr);
			printf ("Warning threshold	: %d BTU/hr\n",warning_threshbtuphr);
			printf ("Failure threshold	: %d BTU/hr\n\n",failuer_thresbtuphr);
		}
		else
		{
		  //printf ("Reading 			: %d W \n",(readingbtuphr));
			printf ("Warning threshold	: %d W \n",(warning_threshbtuphr));
			printf ("Failure threshold	: %d W \n\n",(failuer_thresbtuphr));
		}
    }
	else
	{
		printf ("Error : Can not access the sensor data \n\n");
	}	
    return status;
}
		


static int ipmi_get_instan_power_consmpt_data(struct ipmi_intf* intf,
	IPMI_INST_POWER_CONSUMPTION_DATA* instpowerconsumptiondata)
{

	struct ipmi_rs * rsp;
	struct ipmi_rq req={0};
 
	uint8_t msg_data[2];


	/*get instantaneous power consumption command*/
	req.msg.netfn = 0x30;
	req.msg.lun = 0;
	req.msg.cmd = 0xb3;

	req.msg.data = msg_data;
	req.msg.data_len = 2;


		
	memset(msg_data, 0, 2);

	msg_data[0] = 0x0A;	
	msg_data[1] = 0x00;


	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
				lprintf(LOG_ERR, " Error getting instantaneous power consumption data .\n");
				
				return -1;
	} else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb)) {
		lprintf(LOG_ERR, "  Error getting instantaneous power consumption data: Command not supported on this system.");
		return -1;
	} else if (rsp->ccode != 0){
		lprintf(LOG_ERR, "  Error getting instantaneous power consumption data: %s",
				val2str(rsp->ccode, completion_code_vals));
		
		return -1;
	}

	* instpowerconsumptiondata = * ( (IPMI_INST_POWER_CONSUMPTION_DATA*) (rsp->data));

	//instpowerconsumptiondata->instanpowerconsumption=((rsp->data[1]<<8)+rsp->data[0]);
	//instpowerconsumptiondata->instanApms=((rsp->data[3]<<8)+rsp->data[2]);

return 0;


}
	

uint64_t watt_to_btuphr_conversion(uint32_t powerinwatt)
{
	uint64_t powerinbtuphr;
	/*returning the floor value*/
  
    
       powerinbtuphr=(3.413*powerinwatt);
 // lprintf(LOG_ERR, " power in watt %d \n  power in btuphr %d",
	//						powerinwatt, powerinbtuphr);
       
    return(powerinbtuphr);
    

}
uint32_t btuphr_to_watt_conversion(uint64_t powerinbtuphr)
{
	uint32_t powerinwatt;
	/*returning the floor value*/
	powerinwatt= (powerinbtuphr/3.413);
	return (powerinwatt);
}


static void ipmi_print_get_instan_power_Amps_data(IPMI_INST_POWER_CONSUMPTION_DATA* instpowerconsumptiondata)
{
	uint16_t intampsval=0;
    uint16_t decimalampsval=0;

	
    if (instpowerconsumptiondata->instanApms>0)
    {
     	decimalampsval=(instpowerconsumptiondata->instanApms%10);
        intampsval=instpowerconsumptiondata->instanApms/10;
    }
    printf("\nAmpearage value: %d.%d A \n",intampsval,decimalampsval);
}
static int ipmi_print_get_power_consmpt_data(struct ipmi_intf* intf,uint8_t  unit)
{
		
	int rc = 0;
	int i;
	uint16_t inputwattageL=0;
	int sensorIndex = 0;
	
	uint32_t readingbtuphr;
	uint32_t warning_threshbtuphr;
	uint32_t failuer_thresbtuphr;
	IPMI_INST_POWER_CONSUMPTION_DATA instpowerconsumptiondata;

	printf ("\nPower consumption information\n");
	
	rc=ipmi_get_instan_power_consmpt_data(intf,&instpowerconsumptiondata); if (-1 == rc) return rc;
  
	rc=ipmi_get_power_consumption_data(intf,unit); if (-1 == rc) return rc;
	
    rc=ipmi_get_power_headroom_command(intf,unit);	if (-1 == rc) return rc;
        
	ipmi_print_get_instan_power_Amps_data(&instpowerconsumptiondata);
	return rc;


}
/*ajay1*/
/*ajay2*/
static int ipmi_get_avgpower_consmpt_history(struct ipmi_intf* intf,IPMI_AVGPOWER_CONSUMP_HISTORY* pavgpower )
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];

	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_GET_SYS_INFO;
	req.msg.data_len = 4;
	req.msg.data = data;
	data[0] = 0;
	data[1] = 0xeb;
	data[2] = 0;
	data[3] = 0;

	rsp = intf->sendrecv(intf, &req);

	if (rsp == NULL)
	{
		lprintf(LOG_ERR, " Error getting average power consumption history data .\n");
		return -1;
	} 
	else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb))
	{
		lprintf(LOG_ERR, "  Error getting average power consumption  history data: Command not supported on this system.");
		return -1;
	}
	else if (rsp->ccode != 0)
	{
		lprintf(LOG_ERR, "  Error getting average power consumption historydata: %s",
				val2str(rsp->ccode, completion_code_vals));
     
		return -1;
	}

 	if (verbose > 1)
	{
       printf("Average power consumption history  Data               :%x %x %x %x %x %x %x\n\n",
				rsp->data[0], rsp->data[1], rsp->data[2], rsp->data[3], 
				rsp->data[4], rsp->data[5], rsp->data[6], rsp->data[7]);
												
 	}

	*pavgpower = *( (IPMI_AVGPOWER_CONSUMP_HISTORY*) rsp->data);
	
	/*
	lastminuteavgpower=(rsp->data[2]<<8)+rsp->data[1];
	lasthouravgpower=(rsp->data[4]<<8)+rsp->data[3];
	lastdayavgpower=(rsp->data[6]<<8)+rsp->data[5];
	lastweakavgpower=(rsp->data[8]<<8)+rsp->data[7];
    */               
	return 0;
}

static int ipmi_get_peakpower_consmpt_history(struct ipmi_intf* intf,IPMI_PEAKPOWER_CONSUMP_HISTORY * pstPeakpower)
{
	
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];

	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_GET_SYS_INFO;
	req.msg.data_len = 4;
	req.msg.data = data;
	data[0] = 0;
	data[1] = 0xec;
	data[2] = 0;
	data[3] = 0;

	rsp = intf->sendrecv(intf, &req);

	if (rsp == NULL)
	{
		lprintf(LOG_ERR, " Error getting  peak power consumption history data .\n");
		return -1;
	}
	else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb))
	{
		lprintf(LOG_ERR, "  Error getting peak power consumption history data: Command not supported on this system.");
		return -1;
	}
	else if (rsp->ccode != 0)
	{
		lprintf(LOG_ERR, "  Error getting peak power consumption history data: %s",
				val2str(rsp->ccode, completion_code_vals));
  		return -1;
	}
    
 	if (verbose > 1)
	{
 		printf("Peak power consmhistory  Data               : %x %x %x %x %x %x %x %x %x %x\n	%x %x %x %x %x %x %x %x %x %x %x %x %x\n\n",
 				rsp->data[0], rsp->data[1], rsp->data[2], rsp->data[3], 
 				rsp->data[4], rsp->data[5], rsp->data[6], rsp->data[7], 
 				rsp->data[8], rsp->data[9], rsp->data[10], rsp->data[11], 
 				rsp->data[12], rsp->data[13], rsp->data[14], rsp->data[15], 
 				rsp->data[16], rsp->data[17], rsp->data[18], rsp->data[19],  
 				rsp->data[20], rsp->data[21], rsp->data[22], rsp->data[23]
			);
 
    }
	/*need to llok into */
  	*pstPeakpower =* ((IPMI_PEAKPOWER_CONSUMP_HISTORY*)rsp->data);
	return 0;
}

static int ipmi_print_power_consmpt_history(struct ipmi_intf* intf,int unit )
 {
	
	char timestr[30];
    

	uint32_t lastminutepeakpower;
    uint32_t lasthourpeakpower;
    uint32_t lastdaypeakpower;
    uint32_t lastweekpeakpower;


	IPMI_AVGPOWER_CONSUMP_HISTORY avgpower;

	IPMI_PEAKPOWER_CONSUMP_HISTORY stPeakpower;
	uint64_t tempbtuphrconv;
	uint16_t temp;
    int rc=0;
        
	
	rc= ipmi_get_avgpower_consmpt_history(intf,&avgpower);    

    rc= ipmi_get_peakpower_consmpt_history(intf,&stPeakpower);


   	if(rc==0) 
    {
		/*need to put check condition*/
		lastminutepeakpower=stPeakpower.lastminutepeakpowertime;
		lasthourpeakpower=stPeakpower.lasthourpeakpowertime;
		lastdaypeakpower=stPeakpower.lastdaypeakpowertime;
		lastweekpeakpower=stPeakpower.lastweekpeakpowertime;

		
		printf ("Power consumption history\n\n");
		if (unit ==btuphr)
		{
			tempbtuphrconv=watt_to_btuphr_conversion(avgpower.lastminuteavgpower);
			printf ("last minute average power : %d  BTU/hr\n",tempbtuphrconv);
			tempbtuphrconv=watt_to_btuphr_conversion(avgpower.lasthouravgpower);
			printf ("last hour average power   : %d  BTU/hr\n",tempbtuphrconv);
			tempbtuphrconv=watt_to_btuphr_conversion(avgpower.lastdayavgpower);
			printf ("last day average power    : %d  BTU/hr\n",tempbtuphrconv);
			tempbtuphrconv=watt_to_btuphr_conversion(avgpower.lastweakavgpower);
			printf ("last week average power   : %d  BTU/hr\n",tempbtuphrconv);
			tempbtuphrconv=watt_to_btuphr_conversion(stPeakpower.lastminutepeakpower);
			printf ("last minute peak power    : %d  BTU/hr\n",tempbtuphrconv);
			tempbtuphrconv=watt_to_btuphr_conversion(stPeakpower.lasthourpeakpower);
			printf ("last hour peak  power     : %d  BTU/hr\n",tempbtuphrconv);
			tempbtuphrconv=watt_to_btuphr_conversion(stPeakpower.lastdaypeakpower);
			printf ("last day peak power       : %d  BTU/hr\n",tempbtuphrconv);
			tempbtuphrconv=watt_to_btuphr_conversion(stPeakpower.lastweakpeakpower);			printf ("last week peak power      : %d  BTU/hr\n\n",tempbtuphrconv);
		}
		else
		{
			printf ("last minute average power : %d  W\n",avgpower.lastminuteavgpower);
			printf ("last hour average power   : %d  W\n",avgpower.lasthouravgpower);
			printf ("last day average power    : %d  W\n",avgpower.lastdayavgpower);
			printf ("last week average power   : %d  W\n",avgpower.lastweakavgpower);
			printf ("last minute peak power    : %d  W\n",stPeakpower.lastminutepeakpower);
			printf ("last hour peak power      : %d  W\n",stPeakpower.lasthourpeakpower);
			printf ("last day peak power       : %d  W\n",stPeakpower.lastdaypeakpower);
			printf ("last week peak power      : %d  W\n\n",stPeakpower.lastweakpeakpower);
		}
		ipmi_time_to_str(lastminutepeakpower, timestr);
		printf ("last minute peak power time   : %s \n",timestr);
		ipmi_time_to_str(lasthourpeakpower, timestr);
		printf ("last hour peak power time     : %s \n",timestr);
		ipmi_time_to_str(lastdaypeakpower, timestr);
		printf ("last day peak power time      : %s \n",timestr);
		ipmi_time_to_str(lastweekpeakpower, timestr);
		printf ("last week peak power time     : %s \n",timestr);
    }    
}
/*Power cap command implementation*//*Power cap command implementation*/


static int ipmi_get_power_cap(struct ipmi_intf* intf,IPMI_POWER_CAP* ipmipowercap )
{
	struct ipmi_rs * rsp=NULL;
	struct ipmi_rq req={0};
	uint64_t tempbtuphrconv;
	uint8_t data[4];
      
	/* power supply rating command*/
	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_GET_SYS_INFO;
	req.msg.data_len = 4;
	req.msg.data = data;
	
	data[0] = 0;
	data[1] = IPMI_DELL_POWER_CAP;
	data[2] = 0;
	data[3] = 0;
	
		
	rsp = intf->sendrecv(intf, &req);

    if (rsp == NULL) {
				lprintf(LOG_ERR, " Error getting power cap  .\n");
				if (verbose > 1){
				printf("power cap  Data               :%x %x %x %x %x %x %x %x %x %x ",
												 rsp->data[1], rsp->data[2], rsp->data[3], 
												rsp->data[4], rsp->data[5], rsp->data[6], rsp->data[7], 
												rsp->data[8], rsp->data[9], rsp->data[10],rsp->data[11]);
				}
				return -1;
				
	} else if ((rsp->ccode == 0xc1)||(rsp->ccode == 0xcb)) {
	
		lprintf(LOG_ERR, "  Error getting power cap: Command not supported on this system.");
		if (verbose > 1){
		printf("power cap  Data               :%x %x %x %x %x %x %x %x %x %x ",
												 rsp->data[1], rsp->data[2], rsp->data[3], 
												rsp->data[4], rsp->data[5], rsp->data[6], rsp->data[7], 
												rsp->data[8], rsp->data[9], rsp->data[10],rsp->data[11]);

			}
		return -1;
	} else if (rsp->ccode != 0){
		lprintf(LOG_ERR, "  Error getting power cap: %s",
				val2str(rsp->ccode, completion_code_vals));
		if (verbose > 1){
		printf("power cap  Data               :%x %x %x %x %x %x %x %x %x %x ",
												 rsp->data[1], rsp->data[2], rsp->data[3], 
												rsp->data[4], rsp->data[5], rsp->data[6], rsp->data[7], 
												rsp->data[8], rsp->data[9], rsp->data[10],rsp->data[11]);
			}
		return -1;
	}
		if (verbose > 1){
		printf("power cap  Data               :%x %x %x %x %x %x %x %x %x %x ",
												 rsp->data[1], rsp->data[2], rsp->data[3], 
												rsp->data[4], rsp->data[5], rsp->data[6], rsp->data[7], 
												rsp->data[8], rsp->data[9], rsp->data[10],rsp->data[11]);

		}

//Deepaganesh 
//swap is disabled
//Compiler dependent code.Due to alignment padding it may cause problem in some OS.
	* ipmipowercap = *((IPMI_POWER_CAP*)(rsp->data));

/*
	ipmipowercap->PowerCap=((rsp->data[1]<<8)+rsp->data[2]);
	ipmipowercap->unit=rsp->data[3];
	ipmipowercap->MaximumPowerConsmp=((rsp->data[4]<<8)+rsp->data[5]);
	ipmipowercap->MinimumPowerConsmp=((rsp->data[6]<<8)+rsp->data[7]);
	ipmipowercap->totalnumpowersupp=(rsp->data[8]);
	ipmipowercap->AvailablePower=((rsp->data[9]<<8)+rsp->data[10]);
	ipmipowercap->SystemThrottling=(rsp->data[11]);
*/
	
	return 0;
}
static int ipmi_print_power_cap(struct ipmi_intf* intf,uint8_t unit )
{
	uint64_t tempbtuphrconv;
	int rc;
    IPMI_POWER_CAP ipmipowercap;

	rc=ipmi_get_power_cap(intf,&ipmipowercap);
  
// 	ipmipowercap.MaximumPowerConsmp = BSWAP_16(ipmipowercap.MaximumPowerConsmp);
//	ipmipowercap.MinimumPowerConsmp = BSWAP_16(ipmipowercap.MinimumPowerConsmp);
//	ipmipowercap.PowerCap = BSWAP_16(ipmipowercap.PowerCap);
	
	if (rc==0) 
    {
	if (unit ==btuphr){
		tempbtuphrconv=watt_to_btuphr_conversion(ipmipowercap.MaximumPowerConsmp);
		printf ("Maximum power: %d  BTU/hr\n",tempbtuphrconv);
		tempbtuphrconv=watt_to_btuphr_conversion(ipmipowercap.MinimumPowerConsmp);
		printf ("Minimum power: %d  BTU/hr\n",tempbtuphrconv);
		tempbtuphrconv=watt_to_btuphr_conversion(ipmipowercap.PowerCap);
		printf ("Power cap    : %d  BTU/hr\n",tempbtuphrconv);
	}else{
	
		printf ("Maximum power: %d Watt\n",ipmipowercap.MaximumPowerConsmp);
		printf ("Minimum power: %d Watt\n",ipmipowercap.MinimumPowerConsmp);
		printf ("Power cap    : %d Watt\n",ipmipowercap.PowerCap);
	}
    }
	return rc;

}      
static int ipmi_set_power_cap(struct ipmi_intf* intf,int unit,int val )
{
	struct ipmi_rs *rsp = NULL;
	struct ipmi_rq req={0};;
	uint8_t data[13];
	uint16_t powercapval;
    uint64_t maxpowerbtuphr;
	uint64_t maxpowerbtuphr1;
    uint64_t minpowerbtuphr;
	int rc;
	IPMI_POWER_CAP ipmipowercap;

	ipmi_get_power_capstatus_command(intf);
	if (PowercapSetable_flag!=1)
    {
	    lprintf(LOG_ERR, " Can not set powercap on this system");
	    return -1;
	}
	else if(PowercapstatusFlag!=1)
	{
	    lprintf(LOG_ERR, " Power cap set feature is not enabled");
	    return -1;
    }
	
    req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_GET_SYS_INFO;
	req.msg.data_len = 4;
	memset(data, 0, 4);
    req.msg.data = data;
	
	data[0] = 0;
	data[1] = IPMI_DELL_POWER_CAP;
	data[2] = 0;
	data[3] = 0;
	
	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) 
	{
		lprintf(LOG_ERR, " Error getting power cap  .\n");
		if (verbose > 1)
		{
			printf("power cap  Data               :%x %x %x %x %x %x %x %x %x %x ",
											 rsp->data[1], rsp->data[2], rsp->data[3], 
												rsp->data[4], rsp->data[5], rsp->data[6], rsp->data[7], 
												rsp->data[8], rsp->data[9], rsp->data[10],rsp->data[11]);
		}
		return -1;
			
	}
	else if (rsp->ccode == 0xc1) 
	{
	
		lprintf(LOG_ERR, "  Error getting power cap, command not supported on this system.\n");
		if (verbose > 1){
		printf("power cap  Data               :%x %x %x %x %x %x %x %x %x %x ",
												 rsp->data[1], rsp->data[2], rsp->data[3], 
												rsp->data[4], rsp->data[5], rsp->data[6], rsp->data[7], 
												rsp->data[8], rsp->data[9], rsp->data[10],rsp->data[11]);

			}
		return -1;
	}
	else if (rsp->ccode != 0)
	{
		lprintf(LOG_ERR, "  Error getting power cap: %s",
				val2str(rsp->ccode, completion_code_vals));
		if (verbose > 1)
		{
			printf("power cap  Data               :%x %x %x %x %x %x %x %x %x %x ",
												 rsp->data[1], rsp->data[2], rsp->data[3], 
												rsp->data[4], rsp->data[5], rsp->data[6], rsp->data[7], 
												rsp->data[8], rsp->data[9], rsp->data[10],rsp->data[11]);
		}
		return -1;
	}
	if (verbose > 1)
	{
		printf("power cap  Data               :%x %x %x %x %x %x %x %x %x %x ",
												 rsp->data[1], rsp->data[2], rsp->data[3], 
												rsp->data[4], rsp->data[5], rsp->data[6], rsp->data[7], 
												rsp->data[8], rsp->data[9], rsp->data[10],rsp->data[11]);

	}
//	ipmipowercap = *( (IPMI_POWER_CAP *) rsp->data);
	
	ipmipowercap.PowerCap=((rsp->data[1]<<8)+rsp->data[2]);
	ipmipowercap.unit=rsp->data[3];
	ipmipowercap.MaximumPowerConsmp=((rsp->data[4]<<8)+rsp->data[5]);
	ipmipowercap.MinimumPowerConsmp=((rsp->data[6]<<8)+rsp->data[7]);

	//rc=ipmi_get_power_cap(intf,ipmipowercap);
	memset(data, 0, 13);
	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_SET_SYS_INFO;
	req.msg.data_len = 13;
	req.msg.data = data;
	data[0] = IPMI_DELL_POWER_CAP;
	powercapval=val;
	

	data[1] = (powercapval&0XFF);			// command - custom, default, none
	data[2] = ((powercapval&0XFF00)>>8);
	data[3] = unit;

    data[4]=((ipmipowercap.MaximumPowerConsmp&0xFF));
	data[5]=((ipmipowercap.MaximumPowerConsmp&0xFF00)>>8);
	data[6]=((ipmipowercap.MinimumPowerConsmp&0xFF));
	data[7]=((ipmipowercap.MinimumPowerConsmp&0xFF00)>>8);
	data[8]=(ipmipowercap.totalnumpowersupp);
	data[9]=((ipmipowercap.AvailablePower&0xFF));
	data[10]=((ipmipowercap.AvailablePower&0xFF00)>>8);
	data[11]=(ipmipowercap.SystemThrottling);
	data[12]=0x00;

	ipmipowercap.MaximumPowerConsmp = BSWAP_16(ipmipowercap.MaximumPowerConsmp);
	ipmipowercap.MinimumPowerConsmp = BSWAP_16(ipmipowercap.MinimumPowerConsmp);
	ipmipowercap.PowerCap = BSWAP_16(ipmipowercap.PowerCap);
	if(unit==btuphr)
	{
		val = btuphr_to_watt_conversion(val);
		
	}
	else if(unit ==percent)
	{
       	if((val <1)||(val>100))
        {
    	    lprintf(LOG_ERR, " Cap value is out of boundary conditon it should be between 0  - 100");
            return -1;
        }
		val =( (val*(ipmipowercap.MaximumPowerConsmp -ipmipowercap.MinimumPowerConsmp))/100)+ipmipowercap.MinimumPowerConsmp;
        lprintf(LOG_ERR, " Cap value in percentage is  %d ",val);
		data[1] = (val&0XFF);			
		data[2] = ((val&0XFF00)>>8);
		data[3] = watt;
	}
	if(((val<ipmipowercap.MinimumPowerConsmp)||(val>ipmipowercap.MaximumPowerConsmp))&&(unit==watt))
	{
		lprintf(LOG_ERR, " Cap value is out of boundary conditon it should be between %d  - %d",
							ipmipowercap.MinimumPowerConsmp,ipmipowercap.MaximumPowerConsmp);
		return -1;
	}
    else if(((val<ipmipowercap.MinimumPowerConsmp)||(val>ipmipowercap.MaximumPowerConsmp))&&(unit==btuphr))
	{
        minpowerbtuphr= watt_to_btuphr_conversion(ipmipowercap.MinimumPowerConsmp);
        maxpowerbtuphr=watt_to_btuphr_conversion(ipmipowercap.MaximumPowerConsmp);
        maxpowerbtuphr1= watt_to_btuphr_conversion(ipmipowercap.MaximumPowerConsmp);
        lprintf(LOG_ERR, " Cap value is out of boundary conditon it should be between %d",
							minpowerbtuphr);
         lprintf(LOG_ERR, " -%d",
							maxpowerbtuphr1);
       
		return -1;
	}
	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) 
	{
		lprintf(LOG_ERR, " Error setting power cap");
		return -1;
	}
	else if (rsp->ccode > 0) 
	{
		lprintf(LOG_ERR, " Error setting power cap: %s",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}
	if (verbose > 1)
	{
		printf("CC for setpowercap :%d ",rsp->ccode);
	}
	return 0;
}

uint8_t getpowersupplyfruinfo(struct ipmi_intf *intf, uint8_t id, struct fru_header header,
	struct fru_info fru)
{
struct ipmi_rs * rsp;
	struct ipmi_rq req;
	//struct fru_info fru;
	//struct fru_header header;
	uint8_t msg_data[4];

	memset(&fru, 0, sizeof(struct fru_info));
	memset(&header, 0, sizeof(struct fru_header));

	/*
	 * get info about this FRU
	 */
	memset(msg_data, 0, 4);
	msg_data[0] = id;

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_FRU_INFO;
	req.msg.data = msg_data;
	req.msg.data_len = 1;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		printf(" Device not present (No Response)\n");
		return -1;
	}
	if (rsp->ccode > 0) {
		printf(" Device not present (%s)\n",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}

	fru.size = (rsp->data[1] << 8) | rsp->data[0];
	fru.access = rsp->data[2] & 0x1;

	lprintf(LOG_DEBUG, "fru.size = %d bytes (accessed by %s)",
		fru.size, fru.access ? "words" : "bytes");

	if (fru.size < 1) {
		lprintf(LOG_ERR, " Invalid FRU size %d", fru.size);
		return -1;
	}

	/*
	 * retrieve the FRU header
	 */
	msg_data[0] = id;
	msg_data[1] = 0;
	msg_data[2] = 0;
	msg_data[3] = 8;

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_FRU_DATA;
	req.msg.data = msg_data;
	req.msg.data_len = 4;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		printf(" Device not present (No Response)\n");
		return 1;
	}
	if (rsp->ccode > 0) {
		printf(" Device not present (%s)\n",
		       val2str(rsp->ccode, completion_code_vals));
		return 1;
	}

	if (verbose > 1)
		printbuf(rsp->data, rsp->data_len, "FRU DATA");

	memcpy(&header, rsp->data + 1, 8);



}

int  getpowersupplyoutputwattage(struct ipmi_intf *intf,  int *power_supplywatt1,  int *power_supplywatt2)
{

	struct fru_info pFruInfo;
	uint32_t pRetLocation;
	uint32_t pRetSize;
	IPMISDR*        pRecord;
	int i;
    struct fru_header header1;
	
	struct ipmi_sdr_iterator * itr;
	struct sdr_get_rs * header;
	struct sdr_record_fru_locator * fru;
	int rc, rc1;
    char desc[17];

	//printf("FRU Device Description : Builtin FRU Device (ID 0)\n");
	/* TODO: Figure out if FRU device 0 may show up in SDR records. */
	//rc = ipmi_fru_print(intf, NULL);
	//printf("\n");

	if ((itr = ipmi_sdr_start(intf)) == NULL)
		return -1;

	while ((header = ipmi_sdr_get_next_header(intf, itr)) != NULL)
	{
		if (header->type != SDR_RECORD_TYPE_FRU_DEVICE_LOCATOR)
			continue;

		fru = (struct sdr_record_fru_locator *)
			ipmi_sdr_get_record(intf, header, itr);
        
		if (fru == NULL || !fru->logical)
			continue;
        memset(desc, 0, sizeof(desc));
		memcpy(desc, fru->id_string, fru->id_code & 0x01f);
		desc[fru->id_code & 0x01f] = 0;

		if(strncmp(desc,"PS 1",strlen("PS 1"))==0)
		{
			//printf("FRU Device Description : %s (ID %d)\n", desc, fru->device_id);
			rc=getpowersupplyfruinfo(intf,fru->device_id,header1,pFruInfo);
			rc1=ipmi_fru_get_multirec_location_from_fru(intf,
							fru->device_id,	&pFruInfo,
							& pRetLocation, &pRetSize);
			if (rc1!=0)
			{
				printf(" Unable to get multirec information  for %s\n",desc);continue;
			}
			          

	        *power_supplywatt1=getpowersupplyfrudata( intf, &pFruInfo,
											fru->device_id, pRetLocation);
		    if (*power_supplywatt1<0)
				continue;
		}
		else if (strncmp(desc,"PS 2",strlen("PS 2"))==0)
		{

			//printf("FRU Device Description : %s (ID %d)\n", desc, fru->device_id);
			rc=getpowersupplyfruinfo(intf,fru->device_id,header1,pFruInfo);
			rc1 =ipmi_fru_get_multirec_location_from_fru(intf,
							fru->device_id,
							&pFruInfo,
							& pRetLocation,
					        &pRetSize);
			if (rc1!=0)
			{
				printf(" Unable to get multirec information for %s \n",desc);continue;
			}
			//rc=getpowersupplyfruinfo(intf,fru->device_id,header1,pFruInfo);
			*power_supplywatt2=getpowersupplyfrudata( intf, &pFruInfo,
						fru->device_id, pRetLocation);
			if (*power_supplywatt2<0)
				continue; 
			// printf("input wattage PS2", *power_supplywatt2);
			//printf("Powersupply Output wattage = %d\n",*power_supplywatt2);
		}
	}
	ipmi_sdr_end(intf, itr);
	return 0;
}
// idracvalidator
int
ipmi_idracvalidator_command (struct ipmi_intf * intf)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];

	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_GET_SYS_INFO;
	req.msg.data_len = 4;
	req.msg.data = data;
	data[0] = 0;
	data[1] = IPMI_DELL_IDRAC_VALIDATOR;
	data[2] = 2;
	data[3] = 0;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
	//	lprintf(LOG_ERR, " Error getting idrac validator");
		return -1;
	} else if (rsp->ccode > 0) {
	//	lprintf(LOG_ERR, " Error getting idrac validator: %s",
	//		val2str(rsp->ccode, completion_code_vals));
		return -1;
	}
      if( rsp->data[10]==0x0A)
        {
	iDRAC_FLAG=1;
       
      }
      else
           	iDRAC_FLAG=0;
	return 0;
}


int
ipmi_get_power_capstatus_command (struct ipmi_intf * intf)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[2];

	req.msg.netfn = 0x30;
	req.msg.cmd = IPMI_DELL_POWER_CAP_STATUS;
	req.msg.data_len = 2;
	req.msg.data = data;
	data[0] = 01;
	data[1] = 0xFF;
	
	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		lprintf(LOG_ERR, " Error getting powercap status");
		return -1;
	} else if (rsp->ccode > 0) {
		lprintf(LOG_ERR, " Error getting powercap statusr: %s",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}
    if (rsp->data[0]&0x02)
     PowercapSetable_flag=1;
    if(rsp->data[0]&0x01)
        PowercapstatusFlag=1;
	return 0;
}

int
ipmi_set_power_capstatus_command (struct ipmi_intf * intf,uint8_t val)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[2];
	ipmi_get_power_capstatus_command(intf);

	if (PowercapSetable_flag!=1)
    {
    	lprintf(LOG_ERR, " Can not set powercap on this system");
        return -1;
    }
    req.msg.netfn = 0x30;
	req.msg.cmd = IPMI_DELL_POWER_CAP_STATUS;
	req.msg.data_len = 2;
	req.msg.data = data;
    
	data[0] = 00;
	data[1] = val;
	
	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		lprintf(LOG_ERR, " Error setting powercap status");
		return -1;
	} else if (rsp->ccode > 0) {
		lprintf(LOG_ERR, " Error setting powercap statusr: %s",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}
   
	return 0;
}

static void
ipmi_time_to_str(time_t rawTime, char* strTime)
{
	struct tm * tm;
	char *temp;
	tm = gmtime(&rawTime);
  
	temp = asctime(tm);
  
	strcpy(strTime,temp);
}


static void
ipmi_sel_usage(void)
{
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   sel list");
	lprintf(LOG_NOTICE, "      When used without any arguments, entire contents of SEL are displayed");
	lprintf(LOG_NOTICE, "      first <count>");
	lprintf(LOG_NOTICE, "         Displays first (least recent) count entries in SEL. Count of 0 will display all entries");
	lprintf(LOG_NOTICE, "      last <count>");
	lprintf(LOG_NOTICE, "         Displays last (most recent) count entries in SEL. Count of 0 will display all entries");
	lprintf(LOG_NOTICE, "");
}

static void
ipmi_sensor_usage(void)
{
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   sensor list");
	lprintf(LOG_NOTICE, "      Lists status, reading, and thresholds for sensors");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   sensor get <id>");
	lprintf(LOG_NOTICE, "      Shows information for specified sensor");
	lprintf(LOG_NOTICE, "");
}

static void
ipmi_powermonitor_usage(void)
{
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   powermonitor");
	lprintf(LOG_NOTICE, "      Shows power tracking statistics ");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   powermonitor clear cumulativepower");
	lprintf(LOG_NOTICE, "      Reset cumulative power reading");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   powermonitor clear peakpower");
	lprintf(LOG_NOTICE, "      Reset peak power reading");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   powermonitor powersupplyrating");
	lprintf(LOG_NOTICE, "      Displays individual power supply rating");
	lprintf(LOG_NOTICE, "");
	/*ajay power budget */
	lprintf(LOG_NOTICE, "   powermonitor powerconsumption");
	lprintf(LOG_NOTICE, "      Displays power consumption in <watt|btuphr>");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   powermonitor powerconsumptionhistory <watt|btuphr>");
	lprintf(LOG_NOTICE, "      Displays power consumption history ");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   powermonitor getpowerbudget");
	lprintf(LOG_NOTICE, "      Displays power cap in <watt|btuphr>");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   powermonitor setpowerbudget <val><watt|btuphr|percent>");
	lprintf(LOG_NOTICE, "      Allows user to set the  power cap in <watt|BTU/hr|percentage>");
	lprintf(LOG_NOTICE, "");
       lprintf(LOG_NOTICE, "   powermonitor enablepowercap ");
	lprintf(LOG_NOTICE, "      To enable set power cap");
	lprintf(LOG_NOTICE, "");
       lprintf(LOG_NOTICE, "   powermonitor disablepowercap ");
	lprintf(LOG_NOTICE, "      To disable set power cap");
	lprintf(LOG_NOTICE, "");

}
	
static void
ipmi_sysinfo_usage(void)
{
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   sysinfo");
	lprintf(LOG_NOTICE, "      Show system information");
}

static void
ipmi_lcd_usage(void)
{
	lprintf(LOG_NOTICE, "");
    if(iDRAC_FLAG==0)
        {
	lprintf(LOG_NOTICE, "   lcd set {none}|{default}|{custom <text>}");
	lprintf(LOG_NOTICE, "      Set LCD text displayed during non-fault conditions");
        }
    else if(iDRAC_FLAG==1)
        {
        lprintf(LOG_NOTICE, "   lcd set {mode}|{lcdqualifier}|{errordisplay}");
        lprintf(LOG_NOTICE, "");
        lprintf(LOG_NOTICE, "   lcd set mode {none}|{modelname}|{ipv4address}|{macaddress}|");
        lprintf(LOG_NOTICE, "   {systemname}|{servicetag}|{ipv6address}|{ambienttemp}");
        lprintf(LOG_NOTICE, "   {systemwatt }|{assettag}|{userdefined}<text>");
        lprintf(LOG_NOTICE, "");
        lprintf(LOG_NOTICE, "   lcd set lcdqualifier {watt}|{btuphr}|{celsius}|{fahrenheit}");
        lprintf(LOG_NOTICE, "");
        lprintf(LOG_NOTICE, "   lcd set errordisplay {sel}|{simple}");
        }
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   lcd info");
	lprintf(LOG_NOTICE, "      Show LCD text that is displayed during non-fault conditions");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   lcd set vkvm{active}|{inactive}");
	lprintf(LOG_NOTICE, "   	Set vKVM active and inactive, message will be displayed on lcd"); 
        lprintf(LOG_NOTICE, "	when vKVM is active and vKVM session is in progress");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   lcd set frontpanelaccess {viewandmodify}|{viewonly}|{disabled}");
	lprintf(LOG_NOTICE, "      Set LCD mode to view and modify, view only or disabled ");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   lcd status");
	lprintf(LOG_NOTICE, "   	Show LCD Status for vKVM display<active|inactive>");
	lprintf(LOG_NOTICE, " 	and Front Panel access mode {viewandmodify}|{viewonly}|{disabled} ");
	lprintf(LOG_NOTICE, "");
}

static void
usage(void)
{
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "usage: delloem <command> [option...]");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "commands:");
	lprintf(LOG_NOTICE, "    sysinfo");
	lprintf(LOG_NOTICE, "    lcd");
	lprintf(LOG_NOTICE, "    sel");
	lprintf(LOG_NOTICE, "    sensor");
	lprintf(LOG_NOTICE, "    powermonitor");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "For help on individual commands type:");
	lprintf(LOG_NOTICE, "delloem <command> help");

}
int
ipmi_delloem_main(struct ipmi_intf * intf, int argc, char ** argv)
{
	int rc = 0;
	int current_arg = 0;
    ipmi_idracvalidator_command(intf);
    
	if (argc == 0 || strncmp(argv[0], "help\0", 5) == 0) 
	{
		usage();
		return 0;
	}

	/* ----------------------------------------
	 ipmitool delloem lcd command begins here*/
	if (strncmp(argv[current_arg], "lcd\0", 4) == 0) 
	{
		current_arg++;
		if (argc < current_arg) 
		{
			usage();
			return -1;
		}

		/* ipmitool delloem lcd info*/
		if (argc == 1) 
		{
			//printf("Test 1");
			 ipmi_lcd_usage();
		}
		else if (strncmp(argv[current_arg], "info\0", 5) == 0) 
		{
        	if(iDRAC_FLAG==1)            
				rc = ipmi_lcd_get_info_wh(intf);
            else
                rc = ipmi_lcd_get_info(intf);
		}
		else if (strncmp(argv[current_arg], "status\0", 7) == 0)
		{
        	rc = ipmi_lcd_get_status(intf);
		}
		/* ipmitool delloem lcd set*/
		else if (strncmp(argv[current_arg], "set\0", 4) == 0) 
		{
			uint8_t line_number = 0;
 
			current_arg++;
			if (argc <= current_arg) 
			{
				//printf("Test 2");
				ipmi_lcd_usage();return -1;
			}

			if (strncmp(argv[current_arg], "line\0", 5) == 0) 
			{
				current_arg++;
				if (argc <= current_arg) {usage();return -1;}
				line_number = (uint8_t)strtoul(argv[current_arg], NULL, 0);
				current_arg++;
				if (argc <= current_arg) {usage();return -1;}
			}


			if ((strncmp(argv[current_arg], "mode\0", 5) == 0)&&(iDRAC_FLAG==1)) 
			{
                current_arg++;
                if (argc <= current_arg) 
                {
          			//printf("Test 3");
                	ipmi_lcd_usage();
                    return -1;
                }    
                if (argv[current_arg] == NULL)  
				{
					//printf("Test 4");
					ipmi_lcd_usage();
                	return -1;
                }
 
            	if (strncmp(argv[current_arg], "none\0", 5) == 0) 
				{
					rc = ipmi_lcd_configure_wh (intf, IPMI_DELL_LCD_CONFIG_NONE,0xFF,0XFF, 0, NULL);
				}
				else if (strncmp(argv[current_arg], "modelname\0", 10) == 0) 
				{
	 
					rc = ipmi_lcd_configure_wh (intf, IPMI_DELL_LCD_CONFIG_DEFAULT,0xFF,0XFF, 0, NULL);

				}
				else if (strncmp(argv[current_arg], "userdefined\0", 12) == 0) 
				{
					current_arg++;
					if (argc <= current_arg) 
					{
						//printf("Test 5");
						ipmi_lcd_usage();return -1;
					}
					rc = ipmi_lcd_configure_wh (intf, IPMI_DELL_LCD_CONFIG_USER_DEFINED,0xFF,0XFF, line_number, argv[current_arg]);
				}
				else if (strncmp(argv[current_arg], "ipv4address\0", 12) == 0) 
				{
					rc = ipmi_lcd_configure_wh (intf, IPMI_DELL_LCD_iDRAC_IPV4ADRESS  ,0xFF,0XFF, 0, NULL);

				}
				else if (strncmp(argv[current_arg], "macaddress\0", 11) == 0) 
				{
					rc = ipmi_lcd_configure_wh (intf, IPMI_DELL_LCD_IDRAC_MAC_ADDRESS,0xFF,0XFF, 0, NULL);

				}
				else if (strncmp(argv[current_arg], "systemname\0", 11) == 0) 
				{
					rc = ipmi_lcd_configure_wh (intf, IPMI_DELL_LCD_OS_SYSTEM_NAME,0xFF,0XFF, 0, NULL);

				}
				else if (strncmp(argv[current_arg], "servicetag\0", 11) == 0) 
				{
					rc = ipmi_lcd_configure_wh (intf, IPMI_DELL_LCD_SERVICE_TAG, 0xFF,0XFF,0, NULL);

				}
				else if (strncmp(argv[current_arg], "ipv6address\0", 12) == 0) 
				{
					rc = ipmi_lcd_configure_wh (intf, IPMI_DELL_LCD_iDRAC_IPV6ADRESS  ,0xFF,0XFF, 0, NULL);

				}
				else if (strncmp(argv[current_arg], "ambienttemp\0", 12) == 0) 
				{
					rc = ipmi_lcd_configure_wh (intf, IPMI_DELL_LCD_AMBEINT_TEMP, 0xFF,0XFF,0, NULL);

				}
				else if (strncmp(argv[current_arg], "systemwatt\0", 11) == 0) 
				{
					rc = ipmi_lcd_configure_wh (intf, IPMI_DELL_LCD_SYSTEM_WATTS , 0xFF,0XFF,0, NULL);

				}
				else if (strncmp(argv[current_arg], "assettag\0", 9) == 0) 
				{
					rc = ipmi_lcd_configure_wh (intf, IPMI_DELL_LCD_ASSET_TAG , 0xFF,0XFF,0, NULL);

				}
				else if (strncmp(argv[current_arg], "help\0", 5) == 0) 
				{
					//printf("Test 6");
					ipmi_lcd_usage();
			    }
				else
				{	
					//printf("Test 7");				
					ipmi_lcd_usage();
				}
            }
			//Deepaganesh - Fix
			else if ((strncmp(argv[current_arg], "lcdqualifier\0", 13)== 0) &&(iDRAC_FLAG==1) )
			{
                   
                current_arg++;
                if (argc <= current_arg) 
				{
					//printf("Test 8");
					ipmi_lcd_usage();
					return -1;
				}
                if (argv[current_arg] == NULL) 
				{
					//printf("Test 9");
					ipmi_lcd_usage();
					return -1;
				} 

	            if (strncmp(argv[current_arg], "watt\0", 5) == 0) {

	                
					rc = ipmi_lcd_configure_wh (intf, 0xFF,0x00,0XFF, 0, NULL);
				}
				else if (strncmp(argv[current_arg], "btuphr\0",7) == 0) {
					rc = ipmi_lcd_configure_wh (intf, 0xFF,0x01,0XFF, 0, NULL);

				} else if (strncmp(argv[current_arg], "celsius\0", 8) == 0) {
				rc = ipmi_lcd_configure_wh (intf, 0xFF,0x02,0xFF, 0, NULL);
				} else if (strncmp(argv[current_arg], "fahrenheit", 11) == 0) {
				rc = ipmi_lcd_configure_wh (intf, 0xFF,0x03,0xFF, 0, NULL);

				}else if (strncmp(argv[current_arg], "help\0", 5) == 0) {
					ipmi_lcd_usage();
					//printf("Test 10");
			    }
				else {	
					ipmi_lcd_usage();
					//printf("Test 11");
				}
            }
			else if( (strncmp(argv[current_arg], "errordisplay\0", 13) == 0)&&(iDRAC_FLAG==1)) 
			{

                current_arg++;
                if (argc <= current_arg)
				{
					//printf("Test 12"); 
					ipmi_lcd_usage();
					return -1;
				}
                if (argv[current_arg] == NULL) 
				{ 
					//printf("Test 13");
					ipmi_lcd_usage();
					return -1;
				} 

	            if (strncmp(argv[current_arg], "sel\0", 4) == 0) 
				{
					rc = ipmi_lcd_configure_wh (intf, 0xFF,0xFF,IPMI_DELL_LCD_ERROR_DISP_SEL , 0, NULL);
				}
				else if (strncmp(argv[current_arg], "simple\0", 7) == 0) 
				{
					rc = ipmi_lcd_configure_wh (intf, 0xFF,0xFF,IPMI_DELL_LCD_ERROR_DISP_VERBOSE , 0, NULL);
	                                 
				}
				else if (strncmp(argv[current_arg], "help\0", 5) == 0) 
				{
					//printf("Test 14");
					ipmi_lcd_usage();
				}
				else 
				{	
					ipmi_lcd_usage();
				}
            }

			else if ((strncmp(argv[current_arg], "none\0", 5) == 0)&&(iDRAC_FLAG==0)) 
			{
				rc = ipmi_lcd_configure (intf, IPMI_DELL_LCD_CONFIG_NONE, 0, NULL);
			}
			else if ((strncmp(argv[current_arg], "default\0", 8) == 0)&&(iDRAC_FLAG==0)) 
			{
				rc = ipmi_lcd_configure (intf, IPMI_DELL_LCD_CONFIG_DEFAULT, 0, NULL);

			} 
			else if ((strncmp(argv[current_arg], "custom\0", 7) == 0)&&(iDRAC_FLAG==0))  
			{
				current_arg++;
				if (argc <= current_arg) 
				{
					ipmi_lcd_usage();
					return -1;
				}
				rc = ipmi_lcd_configure (intf, IPMI_DELL_LCD_CONFIG_USER_DEFINED, line_number, argv[current_arg]);
			} 
			
			else if (strncmp(argv[current_arg], "vkvm\0", 5) == 0) 
			{
				current_arg++;
				if (argc <= current_arg) 
				{
					//printf("Test 17");
					ipmi_lcd_usage();
					return -1;
				}

				if (strncmp(argv[current_arg], "active\0", 7) == 0) 
				{
					rc = ipmi_lcd_set_kvm (intf, 1);
				}
				else if (strncmp(argv[current_arg], "inactive\0", 9)==0)
				{
					rc = ipmi_lcd_set_kvm (intf, 0);
						
				}
				else if (strncmp(argv[current_arg], "help\0", 5) == 0) 
				{
					//printf("Test 18");
					ipmi_lcd_usage();
				}
				else 
				{	
			      //printf("Test 19");
					ipmi_lcd_usage();
				}
				
			}
			else if (strncmp(argv[current_arg], "frontpanelaccess\0", 17) == 0) 
			{
				current_arg++;
				if (argc <= current_arg) 
				{
					//printf("Test 20");
					ipmi_lcd_usage();
					return -1;
				}
				if (strncmp(argv[current_arg], "viewandmodify\0", 14) == 0) 
				{
					rc = ipmi_lcd_set_lock (intf, 0);
				}
				else if (strncmp(argv[current_arg], "viewonly\0", 9)==0)
				{
					rc =  ipmi_lcd_set_lock (intf, 1);
						
				}
				else if (strncmp(argv[current_arg], "disabled\0", 9)==0)
				{
					rc =  ipmi_lcd_set_lock (intf, 2);
						
				}/*ajay*/
				else if (strncmp(argv[current_arg], "help\0", 5) == 0) 
				{
					//printf("Test 21");
					ipmi_lcd_usage();
			    }
				else 
				{	
			       //printf("Test 22");
					ipmi_lcd_usage();
				}

			}
			else if( (strncmp(argv[current_arg], "help\0", 5) == 0)&&(iDRAC_FLAG==0))  
			{
				//printf("Test 23");
				ipmi_lcd_usage();
			}
			else 
			{
				//printf("Test 24");
				ipmi_lcd_usage();
				return -1;
			}
		} 
		else 
		{
			//printf("Test 25");
			ipmi_lcd_usage();
			return -1;
		}
	
	}
/* start sysinfo*/	
	else if (strncmp(argv[current_arg], "sysinfo\0", 8) == 0) 
	{
		current_arg++;

		if (argc <= current_arg) {
			rc = ipmi_sysinfo_id(intf, 0);

		} else {
			ipmi_sysinfo_usage();
		}
	/* start sel processing*/
	}
	else if (strncmp(argv[current_arg], "sel\0", 4) == 0) 
	{
		current_arg++;
		if (argc == 1)
		{
			rc = ipmi_sel_print(intf, 0xFFFF,0);
		} 
		else if (strncmp(argv[current_arg], "list\0", 5) == 0)
		{
			current_arg++;
			int count = 0;
			char *countstr = NULL;
			if (argv[current_arg] == NULL)
			{
				rc = ipmi_sel_print(intf, 0xFFFF,0);
			} 
			else if ( (strncmp(argv[current_arg], "first\0", 6) == 0) ||
					  (argc == 3)
					) 
			{
				current_arg++;
				countstr = argv[current_arg];
				if(argc == 3)
				{
					countstr = argv[2];
				}
				if (countstr) 
				{
					if (make_int(countstr,&count) < 0)
					{
						lprintf(LOG_ERR, "Numeric argument required; got '%s'",
								countstr);
						return -1;
					}
				}

				if (count == 0) 
				{
					rc = ipmi_sel_print(intf, 0xFFFF,0);
				}
				else
				{
					rc = ipmi_sel_print(intf, count, 1);
				}
			}
			else if (strncmp(argv[current_arg], "last\0", 5) == 0) 
			{
				current_arg++;
				countstr = argv[current_arg];
				if (countstr)
				{
					if (make_int(countstr,&count) < 0)
					{
						lprintf(LOG_ERR, "Numeric argument required; got '%s'",
								countstr);
						return -1;
					}
				}
				if (count == 0)
				{
					rc = ipmi_sel_print(intf, 0xFFFF,0);
				}
				else
				{
					rc = ipmi_sel_print(intf, count, 2);
				}

			}
			/*AjayDF180787  FT32:ipmitool:Sel list last or first keyword doesn't hold good. */
			else
			{ 
				ipmi_sel_usage();
				return -1;
			}
			
	 	}
		else
		{ 
			ipmi_sel_usage();
			return -1;
		}
	}
	/* Sensor report processing*/
	else if (strncmp(argv[current_arg], "sensor\0", 7) == 0) 
	{
		current_arg++;
		if (argc == 1)
		{
			rc = ipmi_sensor(intf, 0xff);
		}
		else if (strncmp(argv[current_arg], "list\0", 5) == 0)
		{
			rc = ipmi_sensor(intf, 0xff);
		}
		else if (strncmp(argv[current_arg], "get\0", 4) == 0)
		{
			current_arg++;
			if (argv[current_arg] == NULL)
			{
				ipmi_sensor_usage();
				return -1;
			}
			int currIdInt;
			make_int(argv[current_arg],&currIdInt);
            if(currIdInt<0||currIdInt>255)
            {
            	lprintf(LOG_ERR, "Invalid Id the id should be between 0-255\n");
                ipmi_sensor_usage();
            }
			rc = ipmi_sensor_get(intf, argc - 2, &argv[2]);
		}
		else
		{
			ipmi_sensor_usage();
		}
	/*Powermanagement report processing*/
	}
	else if (strncmp(argv[current_arg], "powermonitor\0", 13) == 0) 
	{
		current_arg++;
		if (argc == 1)
		{
			rc = ipmi_powermgmt(intf);
		}
		else if (strncmp(argv[current_arg], "status\0", 7) == 0) 
		{
			rc = ipmi_powermgmt(intf);
		}
		else if (strncmp(argv[current_arg], "clear\0", 6) == 0) 
		{
			current_arg++;
			if (argv[current_arg] == NULL) 
			{
				ipmi_powermonitor_usage();
				return -1;
			}
			else if (strncmp(argv[current_arg], "peakpower\0", 10) == 0) 
			{
				rc = ipmi_powermgmt_clear(intf, 1);
			}
			else if (strncmp(argv[current_arg], "cumulativepower\0", 16) == 0) 
			{
				rc = ipmi_powermgmt_clear(intf, 0);
			}
			else 
			{
				ipmi_powermonitor_usage();
				return -1;
			}
			
		}
		else if (strncmp(argv[current_arg], "powerconsumption\0", 17) == 0) 
		{
			current_arg++;
              
			if (argv[current_arg] == NULL)
			{
              
				rc=ipmi_print_get_power_consmpt_data(intf,watt);
				
			}
			else if (strncmp(argv[current_arg], "watt\0", 5) == 0) 
			{
		           
				rc = ipmi_print_get_power_consmpt_data(intf, watt);
			}
			else if (strncmp(argv[current_arg], "btuphr\0", 7) == 0) 
			{
				rc = ipmi_print_get_power_consmpt_data(intf, btuphr);
			}
			else
			{
				ipmi_powermonitor_usage();
				return -1;
			}
			
		}
		else if (strncmp(argv[current_arg], "powersupplyrating\0", 18) == 0) 
		{
			rc = ipmi_get_power_supply_rating(intf);
		}
		else if (strncmp(argv[current_arg], "powerconsumptionhistory\0", 23) == 0) 
		{
			current_arg++;
			if (argv[current_arg] == NULL)
			{
				rc=ipmi_print_power_consmpt_history(intf,watt);
				
			}
			else if (strncmp(argv[current_arg], "watt\0", 5) == 0) 
			{
				rc = ipmi_print_power_consmpt_history(intf, watt);
			}
			else if (strncmp(argv[current_arg], "btuphr\0", 7) == 0)
			{
				rc = ipmi_print_power_consmpt_history(intf, btuphr);
			}
			else
			{
				ipmi_powermonitor_usage();
				return -1;
			}
			
		}
		else if (strncmp(argv[current_arg], "getpowerbudget\0", 15) == 0)
		{
			current_arg++;
			if (argv[current_arg] == NULL)
			{
				rc=ipmi_print_power_cap(intf,watt);
				
			}
			else if (strncmp(argv[current_arg], "watt\0", 5) == 0)
			{
				rc = ipmi_print_power_cap(intf, watt);
			}
			else if (strncmp(argv[current_arg], "btuphr\0", 7) == 0)
			{
				rc = ipmi_print_power_cap(intf, btuphr);
			}
			else
			{
				ipmi_powermonitor_usage();
				return -1;
			}
			
		}
		else if (strncmp(argv[current_arg], "setpowerbudget\0", 15) == 0)
		{
			current_arg++;
			int val;
			if (argv[current_arg] == NULL)
			{ 
				ipmi_powermonitor_usage();
				return -1;
			}
			make_int(argv[current_arg],&val);
			current_arg++;
			if (argv[current_arg] == NULL)
			{	
				//c=ipmi_set_power_cap(intf,watt,val);
				ipmi_powermonitor_usage();
			}
            else if (strncmp(argv[current_arg], "watt\0", 5) == 0)
			{
				  rc=ipmi_set_power_cap(intf,watt,val);
			}
			else if (strncmp(argv[current_arg], "btuphr\0", 7) == 0)
			{
				rc=ipmi_set_power_cap(intf, btuphr,val);
			}
			else if (strncmp(argv[current_arg], "percent\0", 8) == 0)
			{
			      rc=ipmi_set_power_cap(intf,percent,val);
			}
            else
			{
				ipmi_powermonitor_usage();
				return -1;
			}
			
		}
		else if (strncmp(argv[current_arg], "enablepowercap\0", 15) == 0)
		{
			ipmi_set_power_capstatus_command(intf,1);
		}
			
		else if (strncmp(argv[current_arg], "disablepowercap\0", 16) == 0)
		{
			ipmi_set_power_capstatus_command(intf,0);
		}
		else
		{
			ipmi_powermonitor_usage();
			return -1;
		}
	}
	else if (strncmp(argv[current_arg], "test\0", 5) == 0) 
	{
		rc = ipmi_sdr_build_table(intf);
	}
	else
	{
		usage();
		return -1;
	}

	return rc;
}


uint16_t compareinputwattage(IPMI_POWER_SUPPLY_INFO* powersupplyinfo, uint16_t inputwattage)
{
	if  (	(inputwattage>powersupplyinfo->ratedWatts)||
			(inputwattage==0)
		)
	{
		inputwattage=powersupplyinfo->ratedWatts;
		//printf("power supply wattage Z %d",inputwattage);
	}
	return inputwattage;
}
